using System;
using System.Linq;
using Benday.Npr.Common.Repositories;
using Benday.Npr.Common.DomainModel;
using System.Xml.Linq;
using Benday.Npr.Common;

namespace Benday.Npr.Common
{
    public static class XmlUtility
    {
        public static string GetAttributeValue(XElement element, string attributeName, string defaultReturnValue)
        {
            if (element == null)
                return defaultReturnValue;
            else
            {
                var value = element.Attribute(attributeName);
                if (value == null)
                    return defaultReturnValue;
                else
                    return value.Value;
            }
        }

        public static void SetChildElement(XElement parentElement, 
            string childElementName, 
            double childElementValue)
        {
            SetChildElement(parentElement, childElementName, 
                childElementValue.ToString());
        }

        public static void SetChildElement(XElement parentElement,
            string childElementName,
            DateTime childElementValue)
        {
            SetChildElement(parentElement, childElementName,
                childElementValue.ToString());
        }

        public static void SetChildElement(XElement parentElement, string childElementName, string childElementValue)
        {
            if (parentElement == null)
                throw new ArgumentNullException("parentElement", "parentElement is null.");
            if (String.IsNullOrEmpty(childElementName))
                throw new ArgumentException("childElementName is null or empty.", "childElementName");
            if (String.IsNullOrEmpty(childElementValue))
            {
                childElementValue = String.Empty;
            }

            parentElement.SetElementValue(childElementName, childElementValue);
        }

        public static XElement GetChildElement(XElement parentElement, string childElementName)
        {
            if (parentElement == null)
                throw new ArgumentNullException("parentElement", "parentElement is null.");
            if (String.IsNullOrEmpty(childElementName))
                throw new ArgumentException("childElementName is null or empty.", "childElementName");

            var childElement = parentElement.Descendants(childElementName).FirstOrDefault();
            return childElement;
        }

        public static string GetChildElementValue(XElement parentElement,
            string childElementName,
            string defaultReturnValue)
        {
            if (parentElement == null)
                throw new ArgumentNullException("parentElement", "parentElement is null.");
            if (String.IsNullOrEmpty(childElementName))
                throw new ArgumentException("childElementName is null or empty.", "childElementName");

            XElement childElement = GetChildElement(parentElement, childElementName);

            if (childElement == null)
                return defaultReturnValue;
            else
                return childElement.Value;
        }

        public static DateTime GetChildElementValue(XElement parentElement,
            string childElementName,
            DateTime defaultReturnValue)
        {
            if (parentElement == null)
                throw new ArgumentNullException("parentElement", "parentElement is null.");
            if (String.IsNullOrEmpty(childElementName))
                throw new ArgumentException("childElementName is null or empty.", "childElementName");

            XElement childElement = GetChildElement(parentElement, childElementName);

            if (childElement == null)
                return defaultReturnValue;
            else
            {
                DateTime returnValue;

                if (DateTime.TryParse(childElement.Value, out returnValue) == true)
                {
                    return returnValue;
                }
                else
                {
                    return defaultReturnValue;
                }
            }
        }

        public static int GetChildElementValue(XElement parentElement,
            string childElementName,
            int defaultReturnValue)
        {
            if (parentElement == null)
                throw new ArgumentNullException("parentElement", "parentElement is null.");
            if (String.IsNullOrEmpty(childElementName))
                throw new ArgumentException("childElementName is null or empty.", "childElementName");

            XElement childElement = GetChildElement(parentElement, childElementName);

            if (childElement == null)
                return defaultReturnValue;
            else
            {
                int returnValue;

                if (Int32.TryParse(childElement.Value, out returnValue) == true)
                {
                    return returnValue;
                }
                else
                {
                    return defaultReturnValue;
                }
            }
        }

        public static string GetChildElementValue(XElement parentElement,
            string childElementName,
            string childAttributeName,
            string childAttributeValue,
            string defaultReturnValue)
        {
            if (parentElement == null)
                throw new ArgumentNullException("parentElement", "parentElement is null.");
            if (String.IsNullOrEmpty(childElementName))
                throw new ArgumentException("childElementName is null or empty.", "childElementName");

            var childElement = (
                from temp in parentElement.Descendants(childElementName)
                where temp.HasAttributes == true &&
                temp.Attribute(childAttributeName) != null &&
                temp.Attribute(childAttributeName).Value == childAttributeValue
                select temp).FirstOrDefault();

            if (childElement == null)
                return defaultReturnValue;
            else
                return childElement.Value;
        }

        public static XDocument StringToXDocument(string fromValue)
        {
            return XDocument.Parse(fromValue);
        }        
    }
}
