﻿using System;
using System.Linq;
using Benday.Npr.Common.Repositories;
using Benday.Npr.Common.DomainModel;
using System.Xml.Linq;
using Benday.Npr.Common;

namespace Benday.Npr.DataAccess
{
    public class FeedXmlAdapter : XmlAdapterBase, IFeedXmlAdapter
    {
        public FeedXmlAdapter(IObjectFactory factory)
        {
            if (factory == null)
                throw new ArgumentNullException("factory", "factory is null.");

            m_ObjectFactoryInstance = factory;
        }

        private IObjectFactory m_ObjectFactoryInstance;
        private IObjectFactory ObjectFactoryInstance
        {
            get { return m_ObjectFactoryInstance; }
        }

        public void Adapt(string fromValue, IFeed toValue)
        {
            if (String.IsNullOrEmpty(fromValue))
                throw new ArgumentException("fromValue is null or empty.", "fromValue");
            if (toValue == null)
                throw new ArgumentNullException("toValue", "toValue is null.");

            var doc = StringToXDocument(fromValue);

            toValue.Title = XmlUtility.GetChildElementValue(doc.Root, "title", String.Empty);

            var items = doc.Descendants("item");

            IFeedItem toFeedItem = null;

            foreach (var item in items)
            {
                toFeedItem = ObjectFactoryInstance.Create<IFeedItem>();

                Adapt(item, toFeedItem);

                toValue.FeedItems.Add(toFeedItem);
            }
        }

        public void Adapt(string fromValue, IFeedItem toValue)
        {
            if (String.IsNullOrEmpty(fromValue))
                throw new ArgumentException("fromValue is null or empty.", "fromValue");
            if (toValue == null)
                throw new ArgumentNullException("toValue", "toValue is null.");
            
            var doc = StringToXDocument(fromValue);

            Adapt(doc.Root, toValue);
        }

        private void Adapt(XElement fromValue, IFeedItem toValue)
        {
            if (fromValue == null)
                throw new ArgumentNullException("fromValue", "fromValue is null.");
            if (toValue == null)
                throw new ArgumentNullException("toValue", "toValue is null.");

            toValue.Id = XmlUtility.GetAttributeValue(fromValue, "id", String.Empty);
            toValue.Title = DataAccessUtility.ReplaceUndisplayableHtml(XmlUtility.GetChildElementValue(fromValue, "title", String.Empty));
            toValue.AdditionalInfo = DataAccessUtility.ReplaceUndisplayableHtml(XmlUtility.GetChildElementValue(fromValue, "additionalInfo", String.Empty));
        }        
    }
}
