using System;
using System.Linq;
using Benday.Npr.Common.Repositories;
using Benday.Npr.Common.DomainModel;
using System.Xml.Linq;
using Benday.Npr.Common;

namespace Benday.Npr.DataAccess
{
    public class RssFeedXmlAdapter : XmlAdapterBase, IRssFeedXmlAdapter
    {
        public RssFeedXmlAdapter(IObjectFactory factory)
        {
            if (factory == null)
                throw new ArgumentNullException("factory", "factory is null.");

            m_ObjectFactoryInstance = factory;
        }

        private IObjectFactory m_ObjectFactoryInstance;
        private IObjectFactory ObjectFactoryInstance
        {
            get { return m_ObjectFactoryInstance; }
        }

        private void AssertIsNotNull(object instance, string message)
        {
            if (instance == null)
            {
                throw new InvalidOperationException(message);
            }
        }

        public void Adapt(string fromValue, IStoryList toValue)
        {
            if (String.IsNullOrEmpty(fromValue))
                throw new ArgumentException("fromValue is null or empty.", "fromValue");
            if (toValue == null)
                throw new ArgumentNullException("toValue", "toValue is null.");

            var doc = StringToXDocument(fromValue);

            var channelElement = XmlUtility.GetChildElement(doc.Root, "channel");

            AssertIsNotNull(channelElement, "Channel element does not exist or could not be found.");

            toValue.Title = XmlUtility.GetChildElementValue(channelElement, "title", String.Empty);
            toValue.Teaser = XmlUtility.GetChildElementValue(channelElement, "description", String.Empty);
            toValue.TeaserMini = toValue.Teaser;              

            var items = channelElement.Descendants("item");

            IStory toStory = null;

            foreach (var item in items)
            {
                toStory = ObjectFactoryInstance.Create<IStory>();

                Adapt(item, toStory);

                toValue.Stories.Add(toStory);
            }
        }

        private void Adapt(XElement fromValue, IStory toValue)
        {
            if (fromValue == null)
                throw new ArgumentNullException("fromValue", "fromValue is null.");
            if (toValue == null)
                throw new ArgumentNullException("toValue", "toValue is null.");

            toValue.Title = XmlUtility.GetChildElementValue(fromValue, "title", String.Empty);
            toValue.Teaser = DataAccessUtility.ReplaceUndisplayableHtml(
                XmlUtility.GetChildElementValue(
                fromValue, "description", String.Empty));
            toValue.TeaserMini = toValue.Teaser;

            IParagraph paragraph = ObjectFactoryInstance.Create<IParagraph>();
            paragraph.Text = toValue.Teaser;

            toValue.Paragraphs.Add(paragraph);

            var enclosure = XmlUtility.GetChildElement(fromValue, "enclosure");

            if (enclosure == null)
            {
                toValue.UrlToMp3 = String.Empty;
            }
            else
            {
                toValue.UrlToMp3 = XmlUtility.GetAttributeValue(enclosure, "url", String.Empty);
            }

            var pubDateAsString = XmlUtility.GetChildElementValue(fromValue, "pubDate", String.Empty);

            if (pubDateAsString == null)
            {
                toValue.PublishDate = new DateTime(1970, 1, 1);
                toValue.StoryDate = toValue.PublishDate;
            }
            else
            {
                DateTime temp = DateTime.MinValue;

                if (DateTime.TryParse(pubDateAsString, out temp) == true)
                {
                    toValue.PublishDate = temp;
                }
                else
                {
                    toValue.PublishDate = new DateTime(1970, 1, 1);
                }

                toValue.StoryDate = toValue.PublishDate;
            }
        }
    }
}
