using System;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Benday.Npr.Common.Repositories;
using Benday.Npr.Common.DomainModel;
using System.Xml.Linq;

namespace Benday.Npr.Silverlight.UnitTests.DataAccess
{
    [TestClass]
    public class RssFeedXmlAdapterFixture : UnitTestBase
    {
        public RssFeedXmlAdapterFixture()
        {
        }

        [TestMethod]
        public void RssFeedXmlAdapter_ParseFeedForItsAllPolitics()
        {
            var fromValue = TestDataUtility.RssFeedForItsAllPolitics();

            var adapter = ObjectFactoryInstance.Create<IRssFeedXmlAdapter>();
            var toValue = ObjectFactoryInstance.Create<IStoryList>();

            adapter.Adapt(fromValue, toValue);

            Assert.AreEqual<string>("NPR: It's All Politics Podcast",
                toValue.Title,
                "Title was wrong.");

            Assert.AreEqual<string>("\"It's All Politics\" features Senior Washington Editor Ron Elving and Political Editor Ken Rudin. They dissect the latest news from inside the beltway, along with other political stories from around the country.",
                toValue.Teaser,
                "Teaser was wrong.");

            Assert.AreEqual<string>(toValue.Teaser, toValue.TeaserMini, "Teaser and TeaserMini should be the same value.");

            Assert.IsNotNull(toValue.Stories, "Stories collection was null.");
            Assert.AreEqual<int>(10,
                toValue.Stories.Count,
                "Stories count was wrong.");

            AssertFeedItemStory(toValue,
                0,
                "It's All Politics 122310",
                "It's a special edition of It's All Politics... As the year comes to a close Ken and Ron remember those in politics who died in 2010.   Some on their list include John Murtha chairman of the appropriations sub committee on defense. Robert Byrd the longest serving senator in the history of the senate. Alexander Haig a former Army general who served as White House chief of staff during the Watergate scandal and Secretary of State during the Reagan administration. Dan Rostenkowski, former chairman of Ways and  Means Committee. Charles Mathias Republican liberal was once called \"the conscience of the Senate\" by Mike Mansfield then majority leader. Ted Stevens, who served longer than any other Republican in the history of the senate, and Veteran reporter Daniel Schorr.",
                "Thu, 23 Dec 2010 18:51:17 -0500",
                "http://podcastdownload.npr.org/anon.npr-podcasts/podcast/510068/132295564/npr_132295564.mp3");

            AssertFeedItemStory(toValue,
                9,
                "It's All Politics October 21 2010",
                "The Podcast receives a very special call from the Listener, Aqua Budda floods the Kentucky airwaves, Sharron Angle is struck by colorblindness in Nevada, an ad tells Latinos \"Don't Vote,\" races tighten in Pennsylvania and Wisconsin, and Pelosi says confidently she'll run for Speaker in 2011.",
                "Thu, 21 Oct 2010 20:43:27 -0400",
                "http://podcastdownload.npr.org/anon.npr-podcasts/podcast/510068/130739629/npr_130739629.mp3");
        }

        [TestMethod]
        public void RssFeedXmlAdapter_ParseFeedForNprHourlyNews()
        {
            var fromValue = TestDataUtility.RssFeedForHourlyNews();

            var adapter = ObjectFactoryInstance.Create<IRssFeedXmlAdapter>();
            var toValue = ObjectFactoryInstance.Create<IStoryList>();

            adapter.Adapt(fromValue, toValue);

            Assert.AreEqual<string>("NPR: Hourly News Summary Podcast",
                toValue.Title,
                "Title was wrong.");

            Assert.AreEqual<string>("Five minutes of NPR news, updated hourly.",
                toValue.Teaser,
                "Teaser was wrong.");

            Assert.AreEqual<string>(toValue.Teaser, toValue.TeaserMini, "Teaser and TeaserMini should be the same value.");

            Assert.IsNotNull(toValue.Stories, "Items collection was null.");
            Assert.AreEqual<int>(1,
                toValue.Stories.Count,
                "Stories count was wrong.");

            AssertFeedItemStory(toValue, 
                0, 
                "NPR News: 01-01-2011 10AM ET", 
                "NPR News: 01-01-2011 10AM ET", 
                "Sat, 01 Jan 2011 10:16:16 -0500", 
                "http://podcastdownload.npr.org/anon.npr-podcasts/podcast/500005/132551165/npr_132551165.mp3");
        }

        private static void AssertFeedItemStory(IStoryList list, int index, string expectedTitle, string expectedDesc, string expectedPubDate, string expectedUrl)
        {
            var item = list.Stories[index];

            Assert.IsNotNull(item, "Item was null.");

            Assert.AreEqual<string>(expectedTitle, item.Title, "Item title was wrong.");
            Assert.AreEqual<string>(expectedDesc, item.Teaser, "Teaser was wrong.");
            Assert.AreEqual<string>(item.Teaser, item.TeaserMini, "Teaser and TeaserMini should be the same value.");

            string expectedItemPubDateAsString = expectedPubDate;
            DateTime expectedItemPubDate = DateTime.Parse(expectedItemPubDateAsString);

            Assert.AreEqual<DateTime>(expectedItemPubDate, item.PublishDate, "Item pubDate was wrong.");
            Assert.AreEqual<DateTime>(item.PublishDate, item.StoryDate, "PublishDate and StoryDate should be the same.");

            Assert.AreEqual<string>(expectedUrl, item.UrlToMp3, "Item url was wrong.");            
        }
    }
}
