using System;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Benday.Npr.Common.Repositories;
using Benday.Npr.Common.DomainModel;
using System.Xml.Linq;
using System.Linq;
using Microsoft.Silverlight.Testing;

namespace Benday.Npr.Silverlight.UnitTests.DataAccess
{
    [TestClass]
    [Tag("UndisplayableHtml")]
    public class UnitTestUtilityFixture
    {
        public UnitTestUtilityFixture()
        {
        }

        [TestMethod]
        public void StringContainsUndisplayableHtmlFormatting_NoUndisplayableHtml()
        {
            string checkThisString = "This is some text without undisplayable html.";

            Assert.IsFalse(UnitTestUtility.StringContainsUndisplayableHtmlFormatting(checkThisString));
        }

        [TestMethod]
        public void StringContainsUndisplayableHtmlFormatting_EscapedEmphasisTag_WithContent()
        {
            string checkThisString = "This is some text &lt;em&gt;asdf&lt;/em&gt; more text.";

            Assert.IsTrue(UnitTestUtility.StringContainsUndisplayableHtmlFormatting(checkThisString));
        }

        [TestMethod]
        public void StringContainsUndisplayableHtmlFormatting_EscapedEmphasisTag_WithoutContent()
        {
            string checkThisString = "This is some text &lt;em&gt;&lt;/em&gt; more text.";

            Assert.IsTrue(UnitTestUtility.StringContainsUndisplayableHtmlFormatting(checkThisString));
        }

        [TestMethod]
        public void StringContainsUndisplayableHtmlFormatting_UnescapedEmphasisTag_WithContent()
        {
            string checkThisString = "This is some text <em>asdf</em> more text.";

            Assert.IsTrue(UnitTestUtility.StringContainsUndisplayableHtmlFormatting(checkThisString));
        }

        [TestMethod]
        public void StringContainsUndisplayableHtmlFormatting_UnescapedEmphasisTag_WithoutContent()
        {
            string checkThisString = "This is some text <em></em> more text.";

            Assert.IsTrue(UnitTestUtility.StringContainsUndisplayableHtmlFormatting(checkThisString));
        }

        [TestMethod]
        public void StringContainsUndisplayableHtmlFormatting_UnescapedStrongTag_WithContent()
        {
            string checkThisString = "This is some text <strong>asdf</strong> more text.";

            Assert.IsTrue(UnitTestUtility.StringContainsUndisplayableHtmlFormatting(checkThisString));
        }

        [TestMethod]
        public void StringContainsUndisplayableHtmlFormatting_UnescapedStrongTag_WithoutContent()
        {
            string checkThisString = "This is some text <strong></strong> more text.";

            Assert.IsTrue(UnitTestUtility.StringContainsUndisplayableHtmlFormatting(checkThisString));
        }
    }
}
