﻿using System;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Benday.Npr.Presentation.Interfaces;
using Benday.Npr.Common.DomainModel;
using Benday.Npr.Common;
using Benday.Npr.Common.Repositories;
using Benday.Npr.Silverlight.UnitTests.DataAccess;
using Benday.Npr.Presentation;

namespace Benday.Npr.Silverlight.UnitTests.Presentation
{

    [TestClass]
    public class AllProgramsViewModelFixture : UnitTestBase
    {
        protected override void OnTestInitialize()
        {
            m_FeedDomainServiceInstance = null;
            m_ViewModelInstance = null;
            base.OnTestInitialize();
        }

        protected override void PopulateAdditionalObjectFactoryConfiguration(Factories.ObjectFactoryConfiguration configuration)
        {
            configuration.Instances.Add(typeof(IFeedDomainService), FeedDomainServiceInstance);
        }

        private FeedDomainServiceTester m_FeedDomainServiceInstance;
        public IFeedDomainService FeedDomainServiceInstance
        {
            get
            {
                if (m_FeedDomainServiceInstance == null)
                {
                    m_FeedDomainServiceInstance = new FeedDomainServiceTester();

                    IFeedXmlAdapter adapter = ObjectFactoryInstance.Create<IFeedXmlAdapter>();

                    IFeed feed = ObjectFactoryInstance.Create<IFeed>();

                    adapter.Adapt(TestDataUtility.AllPrograms(), feed);

                    m_FeedDomainServiceInstance.ReturnThisFeed = feed;
                }

                return m_FeedDomainServiceInstance;
            }
        }

        private IAllProgramsViewModel m_ViewModelInstance;
        public IAllProgramsViewModel ViewModelInstance
        {
            get
            {
                if (m_ViewModelInstance == null)
                {
                    m_ViewModelInstance = ObjectFactoryInstance.Create<IAllProgramsViewModel>();
                }

                return m_ViewModelInstance;
            }
        }

        [TestMethod]
        public void AllProgramsViewModel_Load()
        {
            ViewModelInstance.Load();

            Assert.AreEqual<string>("All Programs", ViewModelInstance.Title, "Title was wrong.");

            Assert.IsNotNull(ViewModelInstance.Programs, "Programs was null.");

            Assert.AreEqual<int>(13, ViewModelInstance.Programs.Count, "Program count was wrong.");

            AssertProgram(ViewModelInstance, 0, "All Songs Considered", 37);
            AssertProgram(ViewModelInstance, 1, "All Things Considered", 2);
            AssertProgram(ViewModelInstance, 2, "Fresh Air from WHYY", 13);
            AssertProgram(ViewModelInstance, 3, "JazzSet with Dee Dee Bridgewater", 20);
            AssertProgram(ViewModelInstance, 12, "World of Opera", 36);
        }

        [TestMethod]
        public void AllProgramsViewModel_SelectedItemChange_UpdatesViewModelLocatorAndTriggersSelectedEvent()
        {
            ViewModelLocator.CreateInstance(ObjectFactoryInstance);

            ViewModelInstance.Load();

            Assert.AreEqual<string>("All Programs", ViewModelInstance.Title, "Title was wrong.");

            Assert.IsNotNull(ViewModelInstance.Programs, "Programs was null.");

            Assert.AreEqual<int>(13, ViewModelInstance.Programs.Count, "Program count was wrong.");

            Assert.IsNull(ViewModelInstance.SelectedItem, "Nothing should be selected yet.");

            IProgramViewModel expectedSelection = ViewModelInstance.Programs[0];

            var tester = new AllProgramsViewModelEventTester(ViewModelInstance);

            Assert.IsNull(ViewModelLocator.Instance.CurrentProgram, "CurrentProgramFeed should be null.");

            ViewModelInstance.SelectedItem = expectedSelection;

            Assert.IsTrue(tester.GotSelectedEvent, "Should have gotten selection changed event.");

            Assert.AreSame(expectedSelection, ViewModelLocator.Instance.CurrentProgram, "Wrong instance.");
        }

        public class AllProgramsViewModelEventTester
        {            
            public AllProgramsViewModelEventTester(IAllProgramsViewModel instance)
            {
                if (instance == null)
            		throw new ArgumentNullException("instance", "instance is null.");
            
                m_Instance = instance;

                m_Instance.OnSelectionChanged += new EventHandler(m_Instance_OnProgramSelectionChanged);
            }

            void m_Instance_OnProgramSelectionChanged(object sender, EventArgs e)
            {
                GotSelectedEvent = true;
            }

            private IAllProgramsViewModel m_Instance;
            public IAllProgramsViewModel Instance
            {
                get { return m_Instance; }
            }

            public bool GotSelectedEvent { get; set; }            
        }

        private void AssertProgram(
            IAllProgramsViewModel instance, 
            int programIndex, 
            string expectedTitle, int expectedId)
        {
            var actual = instance.Programs[programIndex];

            Assert.AreEqual<string>(expectedTitle, actual.Title, "Title");
            Assert.AreEqual<string>(expectedId.ToString(), 
                actual.Id, 
                "Id");
        }
    }
}
