using System;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Benday.Npr.Presentation.Interfaces;
using Benday.Npr.Common.DomainModel;
using Benday.Npr.Common;
using Benday.Npr.Common.Repositories;
using Benday.Npr.Silverlight.UnitTests.DataAccess;
using Benday.Npr.Presentation;

namespace Benday.Npr.Silverlight.UnitTests.Presentation
{

    [TestClass]
    public class StoryListViewModelFixture : UnitTestBase
    {
        protected override void OnTestInitialize()
        {
            m_StoryDomainServiceInstance = null;
            m_ViewModelInstance = null;
            base.OnTestInitialize();
        }

        protected override void PopulateAdditionalObjectFactoryConfiguration(
            Factories.ObjectFactoryConfiguration configuration)
        {
            configuration.Instances.Add(typeof(IStoryDomainService), StoryDomainServiceInstance);
        }

        private StoryDomainServiceTester m_StoryDomainServiceInstance;
        public IStoryDomainService StoryDomainServiceInstance
        {
            get
            {
                if (m_StoryDomainServiceInstance == null)
                {
                    m_StoryDomainServiceInstance = new StoryDomainServiceTester();

                    var adapter = ObjectFactoryInstance.Create<IStoryXmlAdapter>();

                    var storyList = ObjectFactoryInstance.Create<IStoryList>();

                    adapter.Adapt(TestDataUtility.AllThingsConsideredStories(), storyList);

                    m_StoryDomainServiceInstance.ReturnThisStoryList = storyList;
                }

                return m_StoryDomainServiceInstance;
            }
        }

        private IStoryListViewModel m_ViewModelInstance;
        public IStoryListViewModel ViewModelInstance
        {
            get
            {
                if (m_ViewModelInstance == null)
                {
                    m_ViewModelInstance = ObjectFactoryInstance.Create<IStoryListViewModel>();
                }

                return m_ViewModelInstance;
            }
        }

        [TestMethod]
        public void StoryListViewModel_Load()
        {
            string programId = "2";

            ViewModelInstance.Load(programId);

            Assert.AreEqual<string>("NPR: All Things Considered", ViewModelInstance.Title, "Title was wrong.");
            Assert.AreEqual<string>(
                "http://www.npr.org/programs/all-things-considered/?ft=3&f=2", 
                ViewModelInstance.UrlToHtml, 
                "UrlToHtml");

            Assert.IsNotNull(ViewModelInstance.Stories, "Stories was null.");

            Assert.AreEqual<int>(10, ViewModelInstance.Stories.Count, "Story count was wrong.");

            AssertProgram(ViewModelInstance, 
                0, 
                "In Israel, When Is A Jew Not Jewish Enough?", "131216486");
            AssertProgram(ViewModelInstance,
                1,
                "Economic Woes Spur Interest In Refinancing Plan", "131223118");
            AssertProgram(ViewModelInstance,
                9,
                "Deficit Panel Pushes Cuts To Social Security", "131223274");
        }

        private void AssertProgram(
            IStoryListViewModel instance, 
            int storyIndex, 
            string expectedTitle,
            string expectedId)
        {
            var actual = instance.Stories[storyIndex];

            Assert.AreEqual<string>(expectedTitle, actual.Title, "Title was wrong.");
            Assert.AreEqual<string>(expectedId, actual.Id, "Id");
        }

        [TestMethod]
        public void AllProgramsViewModel_SelectedItemChange_UpdatesViewModelLocatorAndTriggersSelectedEvent()
        {
            ViewModelLocator.CreateInstance(ObjectFactoryInstance);

            ViewModelInstance.Load("2");

            Assert.AreNotEqual<int>(0, ViewModelInstance.Stories.Count, "Story count was wrong.");

            Assert.IsNull(ViewModelInstance.SelectedItem, "Nothing should be selected yet.");

            var expectedSelection = ViewModelInstance.Stories[0];

            var tester = new StoryListViewModelEventTester(ViewModelInstance);

            Assert.IsNull(ViewModelLocator.Instance.CurrentStory, "CurrentStory should be null.");

            ViewModelInstance.SelectedItem = expectedSelection;

            Assert.IsTrue(tester.GotSelectedEvent, "Should have gotten selection changed event.");

            Assert.AreSame(expectedSelection, ViewModelLocator.Instance.CurrentStory, "Wrong instance.");
        }

        public class StoryListViewModelEventTester
        {
            public StoryListViewModelEventTester(IStoryListViewModel instance)
            {
                if (instance == null)
                    throw new ArgumentNullException("instance", "instance is null.");

                m_Instance = instance;

                m_Instance.OnSelectionChanged += new EventHandler(m_Instance_OnSelectionChanged);
            }

            void m_Instance_OnSelectionChanged(object sender, EventArgs e)
            {
                GotSelectedEvent = true;
            }

            private IStoryListViewModel m_Instance;
            public IStoryListViewModel Instance
            {
                get { return m_Instance; }
            }

            public bool GotSelectedEvent { get; set; }
        }
    }
}
