using System;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Benday.Npr.Presentation.Interfaces;
using Benday.Npr.Common.DomainModel;
using Benday.Npr.Common;
using Benday.Npr.Common.Repositories;
using Benday.Npr.Silverlight.UnitTests.DataAccess;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using Benday.Npr.Presentation;

namespace Benday.Npr.Silverlight.UnitTests.Presentation
{

    [TestClass]
    public class StoryViewModelFixture : UnitTestBase
    {
        public StoryViewModelFixture()
        {
        }

        protected override void OnTestInitialize()
        {
            m_ViewModelInstance = null;
            base.OnTestInitialize();
        }

        private IStory m_StoryModelInstance;
        private IStory StoryModelInstance
        {
            get
            {
                if (m_StoryModelInstance == null)
                {
                    var adapter = ObjectFactoryInstance.Create<IStoryXmlAdapter>();

                    var story = ObjectFactoryInstance.Create<IStory>();

                    adapter.Adapt(TestDataUtility.IndividualStory(), story);

                    m_StoryModelInstance = story;
                }

                return m_StoryModelInstance;
            }
        }

        private IStoryViewModel m_ViewModelInstance;
        public IStoryViewModel ViewModelInstance
        {
            get
            {
                if (m_ViewModelInstance == null)
                {
                    m_ViewModelInstance = ObjectFactoryInstance.Create<IStoryViewModel>();
                }

                return m_ViewModelInstance;
            }
        }

        [TestMethod]
        public void StoryViewModel_Load()
        {
            string programId = "2";

            ViewModelInstance.Load(StoryModelInstance);

            AssertAreEqual(StoryModelInstance, ViewModelInstance);    
        }

        private void AssertAreEqual(IStory expected, IStoryViewModel actual)
        {
            if (expected == null)
                throw new ArgumentNullException("expected", "expected is null.");
            if (actual == null)
                throw new ArgumentNullException("actual", "actual is null.");

            Assert.AreEqual<string>(expected.Id, actual.Id, "Id");
            Assert.AreEqual<string>(expected.UrlToHtml, actual.UrlToHtml, "UrlToHtml");
            Assert.AreEqual<string>(expected.UrlToApi, actual.UrlToApi, "UrlToApi");
            Assert.AreEqual<string>(expected.UrlShort, actual.UrlShort, "UrlShort");
            Assert.AreEqual<string>(expected.Title, actual.Title, "Title");
            Assert.AreEqual<string>(expected.Teaser, actual.Teaser, "Teaser");
            Assert.AreEqual<string>(expected.TeaserMini, actual.TeaserMini, "TeaserMini");
            Assert.AreEqual<string>(expected.Slug, actual.Slug, "Slug");
            Assert.AreEqual<DateTime>(expected.StoryDate, actual.StoryDate, "StoryDate");
            Assert.AreEqual<DateTime>(expected.PublishDate, actual.PublishDate, "PublishDate");
            Assert.AreEqual<string>(expected.ShowName, actual.ShowName, "ShowName");
            Assert.AreEqual<string>(expected.OrganizationName, actual.OrganizationName, "OrganizationName");
            Assert.AreEqual<string>(expected.OrganizationAbbreviation, actual.OrganizationAbbreviation, "OrganizationAbbreviation");
            Assert.AreEqual<string>(expected.OrganizationUrl, actual.OrganizationUrl, "OrganizationUrl");
            Assert.AreEqual<int>(expected.Duration, actual.Duration, "Duration");
            Assert.AreEqual<string>(expected.UrlToMp3, actual.UrlToMp3, "UrlToMp3");
            Assert.AreEqual<string>(expected.UrlToWindowsMedia, actual.UrlToWindowsMedia, "UrlToWindowsMedia");
            Assert.AreEqual<string>(expected.Author, actual.Author, "Author");

            AssertParagraphs(expected.Paragraphs, actual.Paragraphs);
        }

        private void AssertParagraphs(IList<IParagraph> expected, ObservableCollection<IParagraphViewModel> actual)
        {
            if (expected == null)
                throw new ArgumentNullException("expected", "expected is null.");
            if (actual == null)
                throw new ArgumentNullException("actual", "actual is null.");

            Assert.AreEqual<int>(expected.Count, actual.Count, "Count");

            for (int i = 0; i < expected.Count; i++)
            {
                AssertAreEqual(expected[i], actual[i]);
            }
        }

        private void AssertAreEqual(IParagraph expected, IParagraphViewModel actual)
        {
            if (expected == null)
                throw new ArgumentNullException("expected", "expected is null.");
            if (actual == null)
                throw new ArgumentNullException("actual", "actual is null.");

            Assert.AreEqual<string>(expected.Text, actual.Text, "Text");
        }
    }
}
