﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Navigation;
using System.Windows.Shapes;
using Microsoft.Phone.Controls;
using Microsoft.Phone.Shell;
using Benday.Npr.Presentation;
using System.IO.IsolatedStorage;
using System.IO;
using Microsoft.Advertising.Mobile.UI;
using System.Net.NetworkInformation;
using Microsoft.Phone.Net.NetworkInformation;
using Benday.Npr.Common;
using Benday.Npr.Presentation.Interfaces;
using System.Xml.Linq;
using System.Diagnostics;

namespace Benday.Npr.Phone
{
    public partial class App : Application
    {
        private const string STATE_DATA_OBJECT_KEY = "dataObject";
        private const string STATE_DATA_FILENAME = "dataObject.txt";
        
        /// <summary>
        /// Provides easy access to the root frame of the Phone Application.
        /// </summary>
        /// <returns>The root frame of the Phone Application.</returns>
        public PhoneApplicationFrame RootFrame { get; private set; }

        /// <summary>
        /// Constructor for the Application object.
        /// </summary>
        public App()
        {
            // Global handler for uncaught exceptions. 
            UnhandledException += Application_UnhandledException;

            // Show graphics profiling information while debugging.
            if (System.Diagnostics.Debugger.IsAttached)
            {
                // Display the current frame rate counters.
                Application.Current.Host.Settings.EnableFrameRateCounter = true;

                // Show the areas of the app that are being redrawn in each frame.
                //Application.Current.Host.Settings.EnableRedrawRegions = true;

                // Enable non-production analysis visualization mode, 
                // which shows areas of a page that are being GPU accelerated with a colored overlay.
                //Application.Current.Host.Settings.EnableCacheVisualization = true;
            }

            Benday.Npr.Presentation.ViewModelLocator.CreateInstance(new Benday.Npr.Factories.ObjectFactory());

            ViewModelLocator instance = Benday.Npr.Presentation.ViewModelLocator.Instance;

            this.Resources.Add("ViewModelLocatorInstance", instance);

            // Standard Silverlight initialization
            InitializeComponent();

            // Phone-specific initialization
            InitializePhoneApplication();

            // disable screen timeout
            PhoneApplicationService.Current.ApplicationIdleDetectionMode = IdleDetectionMode.Disabled;

            NetworkChange.NetworkAddressChanged += new NetworkAddressChangedEventHandler(NetworkChange_NetworkAddressChanged);

            RefreshIsNetworkAvailable();
        }

        void NetworkChange_NetworkAddressChanged(object sender, EventArgs e)
        {
            RefreshIsNetworkAvailable();
        }

        public void RefreshIsNetworkAvailable()
        {
            NetworkInterfaceType networkInterface = Microsoft.Phone.Net.NetworkInformation.NetworkInterface.NetworkInterfaceType;

            bool isNetworkAvailable = false;

            if (networkInterface == NetworkInterfaceType.MobileBroadbandCdma ||
                networkInterface == NetworkInterfaceType.MobileBroadbandGsm ||
                networkInterface == NetworkInterfaceType.Wireless80211 ||
                networkInterface == NetworkInterfaceType.Ethernet)
            {
                isNetworkAvailable = true;
            }

            ViewModelLocator.Instance.NotifyNetworkAvailabilityChange(isNetworkAvailable);

            if (isNetworkAvailable == false)
            {
                ApplicationUtility.NavigateToMainPage();
            }
        }

        // Code to execute when the application is launching (eg, from Start)
        // This code will not execute when the application is reactivated
        private void Application_Launching(object sender, LaunchingEventArgs e)
        {
        }

        // Code to execute when the application is activated (brought to foreground)
        // This code will not execute when the application is first launched
        private void Application_Activated(object sender, ActivatedEventArgs e)
        {
            /*
            // Check to see if the key for the application state data is in the State dictionary.
            if (PhoneApplicationService.Current.State.ContainsKey(STATE_DATA_OBJECT_KEY))
            {
                // If it exists, assign the data to the application member variable.
                ViewModelLocator.Instance = PhoneApplicationService.Current.State[STATE_DATA_OBJECT_KEY] as ViewModelLocator;                
            }
            */

            RefreshIsNetworkAvailable();

            try
            {
                IDictionary<string, object> stateCollection = PhoneApplicationService.Current.State;

                string resumeAction = stateCollection[ViewModelSerializerBase.SERIALIZATION_KEY_RESUME_ACTION] as string;

                if (resumeAction == ViewModelSerializerBase.SERIALIZATION_KEY_RESUME_ACTION_STORY)
                {
                    new StoryListViewModelSerializer().Deserialize();
                    new StoryViewModelSerializer().Deserialize();
                }
                else if (resumeAction == ViewModelSerializerBase.SERIALIZATION_KEY_RESUME_ACTION_PROGRAM)
                {
                    new StoryListViewModelSerializer().Deserialize();
                    new ProgramViewModelSerializer().Deserialize();
                }
                else if (resumeAction == ViewModelSerializerBase.SERIALIZATION_KEY_RESUME_ACTION_HOURLY_NEWS)
                {
                    new StoryListViewModelSerializer().Deserialize();                    
                }
                else
                {
                    ApplicationUtility.NavigateToMainPage();
                }
            }
            catch (Exception ex)
            {
                Debug.WriteLine(ex.ToString());
                ApplicationUtility.NavigateToMainPage();
            }
        }
        

        // Code to execute when the application is deactivated (sent to background)
        // This code will not execute when the application is closing
        private void Application_Deactivated(object sender, DeactivatedEventArgs e)
        {
            /*
            // Ensure that required application state is persisted here.
            // If there is data in the application member variable...
            if (ViewModelLocator.Instance != null)
            {
                // Store it in the State dictionary.
                PhoneApplicationService.Current.State[STATE_DATA_OBJECT_KEY] = ViewModelLocator.Instance;
            }
            */

            try
            {
                ViewModelSerializerBase.ClearState();

                if (IsDisplayingStory() == true)
                {
                    new StoryListViewModelSerializer().Serialize();
                    new StoryViewModelSerializer().Serialize();

                    ViewModelSerializerBase.SetResumeActionToStory();
                }
                else if (IsDisplayingProgram() == true)
                {
                    new StoryListViewModelSerializer().Serialize();
                    new ProgramViewModelSerializer().Serialize();

                    ViewModelSerializerBase.SetResumeActionToProgram();
                }
                else if (IsDisplayingHourlyNews() == true)
                {
                    new StoryListViewModelSerializer().Serialize();
                    
                    ViewModelSerializerBase.SetResumeActionToHourlyNews();
                }                
            }
            catch (Exception ex)
            {
                Debug.WriteLine(ex.ToString());
            }
        }

        private bool IsDisplayingProgram()
        {
            return ApplicationUtility.DoesCurrentRootVisualSourceContain("StoryListPage.xaml");
        }

        private bool IsDisplayingHourlyNews()
        {
            return ApplicationUtility.DoesCurrentRootVisualSourceContain("HourlyNewsPage.xaml");
        }

        private bool IsDisplayingStory()
        {
            return ApplicationUtility.DoesCurrentRootVisualSourceContain("StoryDetailsPage.xaml");
        }

        // Code to execute when the application is closing (eg, user hit Back)
        // This code will not execute when the application is deactivated
        private void Application_Closing(object sender, ClosingEventArgs e)
        {
        }

        // Code to execute if a navigation fails
        private void RootFrame_NavigationFailed(object sender, NavigationFailedEventArgs e)
        {
            if (System.Diagnostics.Debugger.IsAttached)
            {
                // A navigation has failed; break into the debugger
                System.Diagnostics.Debugger.Break();
            }
        }

        // Code to execute on Unhandled Exceptions
        private void Application_UnhandledException(object sender, ApplicationUnhandledExceptionEventArgs e)
        {
            if (System.Diagnostics.Debugger.IsAttached)
            {
                // An unhandled exception has occurred; break into the debugger
                System.Diagnostics.Debugger.Break();
            }
            else
            {
                MessageBox.Show(e.ExceptionObject.Message);
            }
        }

        #region Phone application initialization

        // Avoid double-initialization
        private bool phoneApplicationInitialized = false;

        // Do not add any additional code to this method
        private void InitializePhoneApplication()
        {
            if (phoneApplicationInitialized)
                return;

            // Create the frame but don't set it as RootVisual yet; this allows the splash
            // screen to remain active until the application is ready to render.
            RootFrame = new TransitionFrame();
            RootFrame.Navigated += CompleteInitializePhoneApplication;

            // Handle navigation failures
            RootFrame.NavigationFailed += RootFrame_NavigationFailed;

            // Ensure we don't initialize again
            phoneApplicationInitialized = true;
        }

        // Do not add any additional code to this method
        private void CompleteInitializePhoneApplication(object sender, NavigationEventArgs e)
        {
            // Set the root visual to allow the application to render
            if (RootVisual != RootFrame)
                RootVisual = RootFrame;

            // Remove this handler since it is no longer needed
            RootFrame.Navigated -= CompleteInitializePhoneApplication;
        }

        #endregion
    }
}