using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using Microsoft.Phone.Controls;
using Benday.Npr.Presentation;
using Microsoft.Phone.Tasks;
using Microsoft.Phone.Shell;
using Benday.Npr.Presentation.Interfaces;
using Microsoft.Xna.Framework.Media;

namespace Benday.Npr.Phone
{
    public class ApplicationBarBindingHelper
    {
        public ApplicationBarBindingHelper(IApplicationBar bar, IMediaPlaybackViewModel viewModel
            )
        {
            if (bar == null)
                throw new ArgumentNullException("bar", "bar is null.");
            if (viewModel == null)
                throw new ArgumentNullException("viewModel", "viewModel is null.");

            m_BarInstance = bar;
            m_ViewModelInstance = viewModel;

            m_ViewModelInstance.PropertyChanged += new System.ComponentModel.PropertyChangedEventHandler(m_ViewModelInstance_PropertyChanged);
            PlayPauseButton.Click += new EventHandler(PlayPauseButton_Click);

            MediaPlayer.MediaStateChanged += new EventHandler<EventArgs>(MediaPlayer_MediaStateChanged);
            
            SyncToMediaPlayer();
        }

        private void SyncToMediaPlayer()
        {
            if (MediaPlayer.GameHasControl == true)
            {
                var state = MediaPlayer.State;

                if (state == MediaState.Stopped)
                {
                    ViewModelInstance.SetState("Stopped");                    
                }
                else if (state == MediaState.Playing)
                {
                    ViewModelInstance.SetState("Playing");                    
                }
                else if (state == MediaState.Paused)
                {
                    ViewModelInstance.SetState("Paused");
                }
            }
            else
            {
                ViewModelInstance.IsPlayable = false;
                BarInstance.IsVisible = false;
            }
        }

        void MediaPlayer_MediaStateChanged(object sender, EventArgs e)
        {
            SyncToMediaPlayer();            
        }

        void PlayPauseButton_Click(object sender, EventArgs e)
        {
            if (MediaPlayer.GameHasControl == true)
            {
                var state = MediaPlayer.State;

                if (state == MediaState.Stopped)
                {
                    // do nothing
                }
                else if (state == MediaState.Playing)
                {
                    MediaPlayer.Pause();                    
                }
                else if (state == MediaState.Paused)
                {
                    MediaPlayer.Resume();
                }
            }
            else
            {
                // do nothing
            }
        }

        void m_ViewModelInstance_PropertyChanged(object sender, System.ComponentModel.PropertyChangedEventArgs e)
        {
            var propertyName = e.PropertyName;

            if (propertyName == null || propertyName == String.Empty)
            {
                return;
            }
            else if (propertyName == "IsPlayable")
            {
                IsPlayableChanged();
            }
            else if (propertyName == "PlayPauseIconUri")
            {
                PlayPauseIconUriChanged();
            }
            else if (propertyName == "PlayPauseText")
            {
                PlayPauseTextChanged();
            }
        }

        private IApplicationBar m_BarInstance;
        public IApplicationBar BarInstance
        {
            get { return m_BarInstance; }
        }

        private IMediaPlaybackViewModel m_ViewModelInstance;
        public IMediaPlaybackViewModel ViewModelInstance
        {
            get { return m_ViewModelInstance; }
        }

        private void IsPlayableChanged()
        {
            BarInstance.IsVisible = ViewModelInstance.IsPlayable;
        }

        private ApplicationBarIconButton PlayPauseButton
        {
            get
            {
                if (BarInstance.Buttons.Count >= 1)
                {
                    return BarInstance.Buttons[0] as ApplicationBarIconButton;
                }
                else
                {
                    return null;
                }
            }
        }

        private void PlayPauseIconUriChanged()
        {
            var button = PlayPauseButton;

            if (button != null)
            {
                button.IconUri = new Uri(ViewModelInstance.PlayPauseIconUri, UriKind.Relative);
            }
        }
        
        private void PlayPauseTextChanged()
        {
            var button = PlayPauseButton;

            if (button != null)
            {
                button.Text = ViewModelInstance.PlayPauseText;
            }
        }
    }
}
