﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using Microsoft.Phone.Controls;
using System.Windows.Navigation;
using Benday.Npr.Presentation;
using Microsoft.Xna.Framework.Media;

namespace Benday.Npr.Phone
{
    public partial class HourlyNewsPage : PhoneApplicationPage
    {
        public HourlyNewsPage()
        {
            InitializeComponent();
        }
        
        private ApplicationBarBindingHelper m_ApplicationBarHelper;
        public ApplicationBarBindingHelper ApplicationBarHelper
        {
            get { return m_ApplicationBarHelper; }
        }

        private Benday.Npr.Presentation.Interfaces.IStoryListViewModel m_ViewModel;

        private void PhoneApplicationPage_Loaded(object sender, RoutedEventArgs e)
        {
            m_ViewModel = ViewModelLocator.Instance.StoryListInstance;

            this.DataContext = m_ViewModel;
        }

        protected override void OnNavigatedTo(NavigationEventArgs e)
        {
            // CheckApplicationBarHelper();

            if (ViewModelLocator.IsInDesignModeStatic == false && ViewModelLocator.Instance.CurrentProgram != null)
            {
                m_ViewModel = ViewModelLocator.Instance.StoryListInstance;

                this.DataContext = m_ViewModel;

                if (ViewModelLocator.Instance.CurrentProgram != null &&
                    ViewModelLocator.Instance.CurrentProgram.Id != m_ViewModel.Id)
                {
                    m_ViewModel.Load(ViewModelLocator.Instance.CurrentProgram.Id);
                }
                
                // CheckApplicationBarHelper();
            }
        }

        private void CheckApplicationBarHelper()
        {
            if (m_ApplicationBarHelper == null)
            {
                m_ApplicationBarHelper = new ApplicationBarBindingHelper(
                    ApplicationBar, 
                    ViewModelLocator.Instance.AllPrograms.MediaPlayback);
            }
        }

        private void PlayTrack()
        {
            try
            {
                if (m_ViewModel != null && m_ViewModel.Stories != null &&
                    m_ViewModel.Stories.Count > 0 && m_ViewModel.Stories[0] != null)
                {
                    Uri uri = new Uri(
                    m_ViewModel.Stories[0].ResolvedUrlToMp3,
                    UriKind.RelativeOrAbsolute);

                    Song song = Song.FromUri(Guid.NewGuid().ToString(), uri);

                    Microsoft.Xna.Framework.Media.MediaPlayer.Play(song);

                    /*
                    var launcher = new MediaPlayerLauncher();

                    launcher.Media = new Uri(ViewModelLocator.Instance.CurrentStory.ResolvedUrlToMp3, UriKind.RelativeOrAbsolute);
                    launcher.Location = MediaLocationType.Data;
                    launcher.Show();
                    */
                }
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message);
            }
        }

        private void CheckPlaybackStateAndOptionallyPlayTrack()
        {
            if (MediaPlayer.GameHasControl == false && MediaPlayer.State == MediaState.Playing)
            {
                var result = MessageBox.Show("Would you like to stop the current music playback?",
                                 "Stop current track?",
                                 MessageBoxButton.OKCancel);

                if (result == MessageBoxResult.OK)
                {
                    PlayTrack();
                }
            }
            else
            {
                PlayTrack();
            }
        }

        void viewModel_OnReadyToPlayMp3(object sender, EventArgs e)
        {
            CheckPlaybackStateAndOptionallyPlayTrack();
        }

        private void Button_Click(object sender, RoutedEventArgs e)
        {
            try
            {
                if (m_ViewModel != null && m_ViewModel.Stories != null &&
                    m_ViewModel.Stories.Count > 0 && m_ViewModel.Stories[0] != null)
                {

                    var storyViewModelInstance = m_ViewModel.Stories[0];

                    // if subscribed to the event, unsubscribe...this ensures that we don't get two events
                    storyViewModelInstance.OnReadyToPlayMp3 -= new EventHandler(viewModel_OnReadyToPlayMp3);

                    // subscribe to the event
                    storyViewModelInstance.OnReadyToPlayMp3 += new EventHandler(viewModel_OnReadyToPlayMp3);

                    storyViewModelInstance.PlayMp3();
                }
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.ToString());
            }
        }
    }
}