﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using Microsoft.Phone.Controls;
using Benday.Npr.Presentation;
using Microsoft.Phone.Tasks;
using Benday.Npr.Presentation.Interfaces;

namespace Benday.Npr.Phone
{
    public partial class MainPage : PhoneApplicationPage
    {
        private Benday.Npr.Presentation.Interfaces.IAllProgramsViewModel m_ViewModel;
        // Constructor
        public MainPage()
        {
            InitializeComponent();
            
            this.Loaded += new RoutedEventHandler(MainPage_Loaded);

            PopulateVersionNumber();
        }

        private void PopulateVersionNumber()
        {
            m_textVersion.Text = "v1.4.2";
        }

        protected override void OnNavigatedTo(System.Windows.Navigation.NavigationEventArgs e)
        {
            // CheckApplicationBarHelper();
            base.OnNavigatedTo(e);
        }

        private void CheckApplicationBarHelper()
        {
            if (m_ApplicationBarHelper == null)
            {
                m_ApplicationBarHelper = new ApplicationBarBindingHelper(this.ApplicationBar, ViewModelLocator.Instance.AllPrograms.MediaPlayback);
            }
        }

        // Handle selection changed on ListBox
        private void MainListBox_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            // If selected index is -1 (no selection) do nothing
            if (MainListBox.SelectedIndex == -1)
                return;

            // Navigate to the new page
            NavigationService.Navigate(new Uri("/StoryListPage.xaml?selectedItem=" + MainListBox.SelectedIndex, UriKind.Relative));

            // Reset selected index to -1 (no selection)
            MainListBox.SelectedIndex = -1;
        }

        private ApplicationBarBindingHelper m_ApplicationBarHelper;
        public ApplicationBarBindingHelper ApplicationBarHelper
        {
            get { return m_ApplicationBarHelper; }
        }

        // Load data for the ViewModel Items
        private void MainPage_Loaded(object sender, RoutedEventArgs e)
        {
            m_ViewModel = ViewModelLocator.Instance.AllPrograms;

            // CheckApplicationBarHelper();

            this.DataContext = m_ViewModel;

            if (m_ViewModel.Programs.Count == 0)
            {
                m_ViewModel.Load();
            }

            m_ViewModel.OnSelectionChanged += new EventHandler(viewModel_OnSelectionChanged);
        }

        void viewModel_OnSelectionChanged(object sender, EventArgs e)
        {
            NavigateToProgram();
        }

        private void NavigateToProgram()
        {
            if (m_ViewModel != null && m_ViewModel.SelectedItem != null)
            {
                if (m_ViewModel.SelectedItem.Id == "RSS_FEED.NPR_HOURLY_NEWS")
                {
                    NavigationService.Navigate(new Uri("/HourlyNewsPage.xaml", UriKind.Relative));   
                }
                else
                {
                    NavigationService.Navigate(new Uri("/StoryListPage.xaml", UriKind.Relative));    
                }                
            }
        }

        private void StackPanel_MouseLeftButtonDown(object sender, MouseButtonEventArgs e)
        {
            
        }

        private void StackPanel_MouseLeftButtonUp(object sender, MouseButtonEventArgs e)
        {
            StackPanel senderAsStackPanel = sender as StackPanel;

            if (senderAsStackPanel != null)
            {
                IProgramViewModel programForThisItem = senderAsStackPanel.DataContext as IProgramViewModel;

                if (programForThisItem != null && 
                    m_ViewModel != null && 
                    m_ViewModel.SelectedItem != null && 
                    m_ViewModel.SelectedItem == programForThisItem)
                {
                    // the current program is being clicked again
                    NavigateToProgram();
                }
            }            
        }

        private void HyperlinkButton_Click(object sender, RoutedEventArgs e)
        {
            HyperlinkButton senderAsHyperLink = sender as HyperlinkButton;

            if (senderAsHyperLink != null)
            {
                var task = new WebBrowserTask();
                task.URL = senderAsHyperLink.Tag as string;

                task.Show();
            }
        }            
    }
}