﻿using System;
using System.Windows;
using System.Windows.Controls;
using Microsoft.Xna.Framework.Media;
using System.Threading;
using Benday.Npr.Presentation;
using Benday.Npr.Presentation.Interfaces;
using Microsoft.Advertising.Mobile.UI;
using System.Windows.Threading;
using Microsoft.Phone.BackgroundAudio;

namespace Benday.Npr.Phone
{
    public partial class MediaPlaybackControl : UserControl
    {
        DispatcherTimer m_timer;

        public MediaPlaybackControl()
        {
            InitializeComponent();
            
            if (GalaSoft.MvvmLight.ViewModelBase.IsInDesignModeStatic == false)
            {
                m_timer = new DispatcherTimer();
                m_timer.Interval = TimeSpan.FromSeconds(0.5);
                m_timer.Tick += new EventHandler(UpdateState);

                BackgroundAudioPlayer.Instance.PlayStateChanged += new EventHandler(Instance_PlayStateChanged);

                m_ButtonImage.DataContext = ViewModelInstance;

                bool isInTestMode = false;

                if (isInTestMode == false)
                {
                    m_adControl.AdUnitId = "66946";
                    m_adControl.ApplicationId = "d38cc7ed-c6b4-4d0c-8595-45bb22eb2801";
                }
                else
                {
                    m_adControl.AdUnitId="Image480_80";
                    m_adControl.ApplicationId = "test_client";
                }
            }
        }

        private void UpdateState(object sender, EventArgs e)
        {
            //txtState.Text = string.Format("State: {0}", BackgroundAudioPlayer.Instance.PlayerState);

            if (BackgroundAudioPlayer.Instance.Track != null)
            {
                //txtTrack.Text = string.Format("Track: {0}", BackgroundAudioPlayer.Instance.Track.Title);

                // Set the current position on the ProgressBar.
                m_ProgressBar.Value = BackgroundAudioPlayer.Instance.Position.TotalSeconds;

                // Update the current playback position.
                TimeSpan position = new TimeSpan();
                position = BackgroundAudioPlayer.Instance.Position;
                m_LabelCurrentPosition.Text = String.Format("{0:d2}:{1:d2}:{2:d2}", position.Hours, position.Minutes, position.Seconds);

                
            }
        }

        void Instance_PlayStateChanged(object sender, EventArgs e)
        {
            switch (BackgroundAudioPlayer.Instance.PlayerState)
            {
                case PlayState.Playing:
                    // Update the UI.
                    m_ProgressBar.IsIndeterminate = false;
                    m_ProgressBar.Maximum = BackgroundAudioPlayer.Instance.Track.Duration.TotalSeconds;
                    //UpdateButtons(true, false, true, true);
                    UpdateState(null, null);

                    // Start the timer for updating the UI.
                    m_timer.Start();
                    break;

                case PlayState.Paused:
                    // Update the UI.
                    //UpdateButtons(true, true, false, true);
                    UpdateState(null, null);

                    // Stop the timer for updating the UI.
                    m_timer.Stop();
                    break;
            }
        }

        private IMediaPlaybackViewModel ViewModelInstance
        {
            get
            {
                return ViewModelLocator.Instance.AllPrograms.MediaPlayback;
            }
        }

        private void SyncToMediaPlayer()
        {
            if (MediaPlayer.GameHasControl == true)
            {
                PlayState state = BackgroundAudioPlayer.Instance.PlayerState;

                if (state == PlayState.Stopped)
                {
                    ViewModelInstance.SetState("Stopped");

                    m_PlaybackContentControl.IsEnabled = false;
                }
                else if (state == PlayState.Playing)
                {
                    ViewModelInstance.SetState("Playing");

                    m_PlaybackContentControl.IsEnabled = true;
                }
                else if (state == PlayState.Paused)
                {
                    ViewModelInstance.SetState("Paused");

                    m_PlaybackContentControl.IsEnabled = true;
                }
            }
            else
            {
                ViewModelInstance.IsPlayable = false;

                m_PlaybackContentControl.IsEnabled = false;
            }
        }

        private void SyncUi(object sender, EventArgs e)
        {
            SyncUi();
        }

        private void SyncUi()
        {
            if (MediaPlayer.GameHasControl == true)
            {
                m_LabelCurrentPosition.Text = new DateTime(MediaPlayer.PlayPosition.Ticks).ToString("m:ss");
            }

            SyncToMediaPlayer();
        }

        private void Play_Click(object sender, RoutedEventArgs e)
        {
            HandlePlayPauseClick();
        }

        private void Pause_Click(object sender, RoutedEventArgs e)
        {
            HandlePlayPauseClick();
        }

        private void HandlePlayPauseClick()
        {
            if (MediaPlayer.GameHasControl == true)
            {
                PlayState state = BackgroundAudioPlayer.Instance.PlayerState;

                if (state == PlayState.Stopped)
                {
                    // do nothing
                }
                else if (state == PlayState.Playing)
                {
                    BackgroundAudioPlayer.Instance.Pause();
                }
                else if (state == PlayState.Paused)
                {
                    BackgroundAudioPlayer.Instance.Play();
                }
            }
            else
            {
                // do nothing
            }
        }
    }
}
