using System;
using System.Collections.Generic;
using System.Linq;
using Benday.Npr.Presentation;
using Microsoft.Phone.Shell;
using Benday.Npr.Presentation.Interfaces;
using Benday.Npr.Common;
using System.Xml.Linq;

namespace Benday.Npr.Phone
{
    public class ProgramViewModelSerializer : ViewModelSerializerBase
    {
        public ProgramViewModelSerializer()
        {

        }

        public void Deserialize()
        {
            if (HasStateValue(ViewModelSerializerBase.SERIALIZATION_KEY_PROGRAM) == false)
            {
                throw new InvalidOperationException("Could not find serialized program xml in state.");
            }
            else
            {
                try
                {
                    System.Text.StringBuilder builder = new System.Text.StringBuilder();
                    string serializedProgramXml = GetStateValue(
                        ViewModelSerializerBase.SERIALIZATION_KEY_PROGRAM);

                    if (String.IsNullOrEmpty(serializedProgramXml) == true)
                    {
                        throw new InvalidOperationException("Serialized program xml was null or empty.");
                    }

                    var document = XmlUtility.StringToXDocument(serializedProgramXml);

                    IProgramViewModel program =
                        ViewModelLocator.Instance.ObjectFactoryInstance.Create<IProgramViewModel>();
                        
                    PopulateFromDocument(document, program);

                    ViewModelLocator.Instance.CurrentProgram = ViewModelLocator.Instance.CurrentProgram;
                }
                catch (Exception)
                {
                    throw;
                }
            }
        }


        private void PopulateFromDocument(XDocument document, IProgramViewModel toValue)
        {
            var root = document.Root;

            toValue.Id = XmlUtility.GetChildElementValue(root, "Id", String.Empty);
            toValue.AdditionalInformation = XmlUtility.GetChildElementValue(root, "AdditionalInformation", String.Empty);
            toValue.Title = XmlUtility.GetChildElementValue(root, "Title", String.Empty);
        }

        public void Serialize()
        {
            if (ViewModelLocator.Instance.CurrentProgram != null)
            {
                Serialize(ViewModelLocator.Instance.CurrentProgram);                
            }            
        }

        private void Serialize(IProgramViewModel fromValue)
        {
            var document = XmlUtility.StringToXDocument("<program />");

            WriteToDocument(document, fromValue);

            SetStateValue(SERIALIZATION_KEY_PROGRAM, document.ToString());
        }

        private void WriteToDocument(System.Xml.Linq.XDocument document,
            IProgramViewModel story)
        {
            var root = document.Root;

            XmlUtility.SetChildElement(root, "Id", story.Id);
            XmlUtility.SetChildElement(root, "Title", story.Title);
            XmlUtility.SetChildElement(root, "AdditionalInformation", story.AdditionalInformation);
        }
    }
}
