using System;
using System.Collections.Generic;
using System.Linq;
using Benday.Npr.Presentation;
using Microsoft.Phone.Shell;
using Benday.Npr.Presentation.Interfaces;
using Benday.Npr.Common;
using System.Xml.Linq;

namespace Benday.Npr.Phone
{
    public class StoryListViewModelSerializer : ViewModelSerializerBase
    {
        public void Deserialize()
        {
            if (HasStateValue(ViewModelSerializerBase.SERIALIZATION_KEY_STORY_LIST) == false)
            {
                throw new InvalidOperationException("Could not find story list xml in state.");
            }
            else
            {
                try
                {
                    System.Text.StringBuilder builder = new System.Text.StringBuilder();
                    string serializedStoryListXml = GetStateValue(
                        ViewModelSerializerBase.SERIALIZATION_KEY_STORY_LIST);

                    if (String.IsNullOrEmpty(serializedStoryListXml) == true)
                    {
                        throw new InvalidOperationException("Story list xml is null or empty.");
                    }

                    var document = XmlUtility.StringToXDocument(serializedStoryListXml);

                    IStoryListViewModel storyList = 
                        ViewModelLocator.Instance.StoryListInstance;

                    PopulateFromDocument(document, storyList);
                }
                catch (Exception)
                {
                    throw;
                }
            }
        }

        private void PopulateFromDocument(XDocument document, IStoryListViewModel toValue)
        {
            var root = document.Root;

            toValue.Id = XmlUtility.GetChildElementValue(root, "Id", String.Empty);
            toValue.UrlToHtml = XmlUtility.GetChildElementValue(root, "UrlToHtml", String.Empty);
            toValue.Title = XmlUtility.GetChildElementValue(root, "Title", String.Empty);
            
            IEnumerable<XElement> stories = root.Elements("story");

            var serializer = new StoryViewModelSerializer();

            IStoryViewModel toValueStory;

            var factory = ViewModelLocator.Instance.ObjectFactoryInstance;

            foreach (var fromValue in stories)
            {
                toValueStory = factory.Create<IStoryViewModel>();

                serializer.Deserialize(fromValue, toValueStory);

                toValue.Stories.Add(toValueStory);
            }
        }

        public void Serialize()
        {
            if (ViewModelLocator.Instance.StoryListInstance != null)
            {
                IStoryListViewModel stories = ViewModelLocator.Instance.StoryListInstance;

                Serialize(stories);
            }            
        }

        private void Serialize(IStoryListViewModel fromValue)
        {
            var document = XmlUtility.StringToXDocument("<stories />");

            WriteToDocument(document, fromValue);

            SetStateValue(SERIALIZATION_KEY_STORY_LIST, document.ToString());
        }

        private void WriteToDocument(System.Xml.Linq.XDocument document,
            IStoryListViewModel storyList)
        {
            var root = document.Root;

            root.SetElementValue("Id", storyList.Id);
            root.SetElementValue("Title", storyList.Title);
            root.SetElementValue("UrlToHtml", storyList.UrlToHtml);

            var storySerializer = new StoryViewModelSerializer();

            foreach (var fromValue in storyList.Stories)
            {
                root.Add(storySerializer.SerializeToElement(fromValue));
            }
        }
    }
}
