using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.Phone.Shell;
using Benday.Npr.Presentation;
using Benday.Npr.Common;
using Benday.Npr.Presentation.Interfaces;
using System.Xml.Linq;

namespace Benday.Npr.Phone
{
    public class StoryViewModelSerializer : ViewModelSerializerBase
    {
        public StoryViewModelSerializer()
        {

        }

        public void Serialize()
        {
            if (ViewModelLocator.Instance.CurrentStory != null)
            {
                var story = ViewModelLocator.Instance.CurrentStory;

                Serialize(story);
            }            
        }

        public XElement SerializeToElement(IStoryViewModel serializeThis)
        {
            return WriteToElement(serializeThis, new XElement("story"));
        }

        private void Serialize(Benday.Npr.Presentation.Interfaces.IStoryViewModel story)
        {
            var document = XmlUtility.StringToXDocument("<story />");

            WriteToDocument(document, story);

            SetStateValue(SERIALIZATION_KEY_STORY, document.ToString());
        }

        private void PopulateStoryFromDocument(XDocument document, IStoryViewModel toValue)
        {
            var fromValue = document.Root;

            Deserialize(fromValue, toValue);
        }

        private void WriteToDocument(System.Xml.Linq.XDocument document,
            IStoryViewModel story)
        {
            var root = document.Root;

            WriteToElement(story, root);
        }

        private XElement WriteToElement(IStoryViewModel story, XElement toElement)
        {
            XmlUtility.SetChildElement(toElement, "Id", story.Id);
            XmlUtility.SetChildElement(toElement, "UrlToHtml", story.UrlToHtml);
            XmlUtility.SetChildElement(toElement, "UrlToApi", story.UrlToApi);
            XmlUtility.SetChildElement(toElement, "UrlShort", story.UrlShort);
            XmlUtility.SetChildElement(toElement, "Title", story.Title);
            XmlUtility.SetChildElement(toElement, "Teaser", story.Teaser);
            XmlUtility.SetChildElement(toElement, "TeaserMini", story.TeaserMini);
            XmlUtility.SetChildElement(toElement, "Slug", story.Slug);
            XmlUtility.SetChildElement(toElement, "StoryDate", story.StoryDate);
            XmlUtility.SetChildElement(toElement, "PublishDate", story.PublishDate);
            XmlUtility.SetChildElement(toElement, "ShowName", story.ShowName);
            XmlUtility.SetChildElement(toElement, "ShowDate", story.ShowDate);
            XmlUtility.SetChildElement(toElement, "OrganizationName", story.OrganizationName);
            XmlUtility.SetChildElement(toElement, "OrganizationAbbreviation", story.OrganizationAbbreviation);
            XmlUtility.SetChildElement(toElement, "OrganizationUrl", story.OrganizationUrl);
            XmlUtility.SetChildElement(toElement, "Duration", story.Duration);
            XmlUtility.SetChildElement(toElement, "UrlToMp3", story.UrlToMp3);
            XmlUtility.SetChildElement(toElement, "UrlToWindowsMedia", story.UrlToWindowsMedia);
            XmlUtility.SetChildElement(toElement, "Author", story.Author);

            XElement paragraphs = new XElement("Paragraphs");

            toElement.Add(paragraphs);

            foreach (var currentParagraph in story.Paragraphs)
            {
                paragraphs.Add(new XElement("Paragraph", currentParagraph.Text));
            }

            return toElement;
        }
        
        public void Deserialize(XElement fromValue, IStoryViewModel toValue)
        {
            toValue.Id = XmlUtility.GetChildElementValue(fromValue, "Id", String.Empty);
            toValue.UrlToHtml = XmlUtility.GetChildElementValue(fromValue, "UrlToHtml", String.Empty);
            toValue.UrlToApi = XmlUtility.GetChildElementValue(fromValue, "UrlToApi", String.Empty);
            toValue.UrlShort = XmlUtility.GetChildElementValue(fromValue, "UrlShort", String.Empty);
            toValue.Title = XmlUtility.GetChildElementValue(fromValue, "Title", String.Empty);
            toValue.Teaser = XmlUtility.GetChildElementValue(fromValue, "Teaser", String.Empty);
            toValue.TeaserMini = XmlUtility.GetChildElementValue(fromValue, "TeaserMini", String.Empty);
            toValue.Slug = XmlUtility.GetChildElementValue(fromValue, "Slug", String.Empty);
            toValue.StoryDate = XmlUtility.GetChildElementValue(fromValue, "StoryDate", default(DateTime));
            toValue.PublishDate = XmlUtility.GetChildElementValue(fromValue, "PublishDate", default(DateTime));
            toValue.ShowName = XmlUtility.GetChildElementValue(fromValue, "ShowName", String.Empty);
            toValue.ShowDate = XmlUtility.GetChildElementValue(fromValue, "ShowDate", default(DateTime));
            toValue.OrganizationName = XmlUtility.GetChildElementValue(fromValue, "OrganizationName", String.Empty);
            toValue.OrganizationAbbreviation = XmlUtility.GetChildElementValue(fromValue, "OrganizationAbbreviation", String.Empty);
            toValue.OrganizationUrl = XmlUtility.GetChildElementValue(fromValue, "OrganizationUrl", String.Empty);
            toValue.Duration = XmlUtility.GetChildElementValue(fromValue, "Duration", default(int));
            toValue.UrlToMp3 = XmlUtility.GetChildElementValue(fromValue, "UrlToMp3", String.Empty);
            toValue.UrlToWindowsMedia = XmlUtility.GetChildElementValue(fromValue, "UrlToWindowsMedia", String.Empty);
            toValue.Author = XmlUtility.GetChildElementValue(fromValue, "Author", String.Empty);

            IEnumerable<XElement> paragraphs = fromValue.Elements("Paragraphs");

            IParagraphViewModel toParagraph;
            IObjectFactory objectFactoryInstance = ViewModelLocator.Instance.ObjectFactoryInstance;

            foreach (var currentParagraph in paragraphs)
            {
                toParagraph = objectFactoryInstance.Create<IParagraphViewModel>();

                toParagraph.Text = currentParagraph.Value;

                toValue.Paragraphs.Add(toParagraph);
            }
        }

        public void Deserialize()
        {
            if (HasStateValue(ViewModelSerializerBase.SERIALIZATION_KEY_STORY) == false)
            {
                throw new InvalidOperationException("Could not find value for serialized story xml.");
            }
            else
            {
                try
                {
                    System.Text.StringBuilder builder = new System.Text.StringBuilder();

                    string serializedStoryXml = GetStateValue(ViewModelSerializerBase.SERIALIZATION_KEY_STORY);

                    if (String.IsNullOrEmpty(serializedStoryXml) == true)
                    {
                        throw new InvalidOperationException("Serialized story xml was null or empty.");
                    }

                    var document = XmlUtility.StringToXDocument(serializedStoryXml);

                    IStoryViewModel story = ViewModelLocator.Instance.ObjectFactoryInstance.Create<IStoryViewModel>();

                    PopulateStoryFromDocument(document, story);

                    ViewModelLocator.Instance.CurrentStory = story;
                }
                catch (Exception)
                {
                    throw;
                }
            }
        }
    }
}
