using System;
using GalaSoft.MvvmLight;
using System.Windows.Input;
using System.Collections.ObjectModel;
using System.Collections.Generic;
using Benday.Npr.Common;
using Benday.Npr.Presentation.Interfaces;
using Benday.Npr.Common.DomainModel;

namespace Benday.Npr.Presentation
{
    public class AllProgramsViewModel : ObjectFactoryViewModel, IAllProgramsViewModel
    {
        public AllProgramsViewModel(IFeedDomainService service, IObjectFactory factory) : base(factory)
        {
            if (service == null)
                throw new ArgumentNullException("service", "service is null.");

            m_Programs = new ObservableCollection<IProgramViewModel>();

            m_ServiceInstance = service;
        }

        private IFeedDomainService m_ServiceInstance;
        private IFeedDomainService ServiceInstance
        {
            get { return m_ServiceInstance; }
        }

        public void Load()
        {
            ServiceInstance.GetAllPrograms(new ReturnResult<IFeed>(FeedReturned));
        }

        private void FeedReturned(ReturnResult<IFeed> result)
        {
            if (result.Error == null && result.Result != null)
            {
                Title = result.Result.Title;
                
                PopulatePrograms(result.Result.FeedItems);
            }
        }

        private void PopulatePrograms(IList<IFeedItem> feedItems)
        {
            this.Programs.Clear();

            IProgramViewModel toValue = null;

            foreach (var fromValue in feedItems)
            {
                toValue = ObjectFactoryInstance.Create<IProgramViewModel>();

                toValue.Title = fromValue.Title;
                toValue.Id = fromValue.Id;
                toValue.AdditionalInformation = fromValue.AdditionalInfo;

                this.Programs.Add(toValue);
            }   
        }

        private const string TitlePropertyName = "Title";

        private string m_Title;
        public string Title
        {
            get
            {
                return m_Title;
            }
            set
            {
                m_Title = value;
                RaisePropertyChanged(TitlePropertyName);
            }
        }

        private ObservableCollection<IProgramViewModel> m_Programs;
        public ObservableCollection<IProgramViewModel> Programs
        {
            get { return m_Programs; }
            set
            {
            	m_Programs = value;
            }
                
        }


        private const string SelectedItemPropertyName = "SelectedItem";

        private IProgramViewModel m_SelectedItem;
        public IProgramViewModel SelectedItem
        {
            get
            {
                return m_SelectedItem;
            }
            set
            {
                if (m_SelectedItem != value)
                {
                    m_SelectedItem = value;

                    ViewModelLocator.Instance.CurrentProgram = m_SelectedItem;

                    RaisePropertyChanged(SelectedItemPropertyName);

                    if (OnSelectionChanged != null)
                    {
                        OnSelectionChanged(this, new EventArgs());
                    }
                }
            }
        }        

        public event EventHandler OnSelectionChanged;

        private IMediaPlaybackViewModel m_MediaPlayback;
        public IMediaPlaybackViewModel MediaPlayback
        {
            get
            {
                if (m_MediaPlayback == null)
                {
                    m_MediaPlayback = ObjectFactoryInstance.Create<IMediaPlaybackViewModel>();
                }

                return m_MediaPlayback;
            }
        }
    }
}
