using System;
using GalaSoft.MvvmLight;
using System.Windows.Input;
using System.Collections.ObjectModel;
using System.Collections.Generic;
using Benday.Npr.Common;
using Benday.Npr.Presentation.Interfaces;
using Benday.Npr.Common.DomainModel;
using System.Windows;

namespace Benday.Npr.Presentation
{
    public class MediaPlaybackViewModel : NetworkAvailabilityViewModel, IMediaPlaybackViewModel
    {
        public MediaPlaybackViewModel()
        {
            IsPlayable = true;
        }

        public bool IsLightThemeEnabled()
        {
            Visibility lightTheme = (Visibility)Application.Current.Resources["PhoneLightThemeVisibility"];

            if (lightTheme == System.Windows.Visibility.Visible)
            {
                return true;
            }
            else
            {
                return false;
            }
        }

        private const string IsPlayablePropertyName = "IsPlayable";

        private bool m_IsPlayable;
        public bool IsPlayable
        {
            get
            {
                return m_IsPlayable;
            }
            set
            {
                m_IsPlayable = value;
                RaisePropertyChanged(IsPlayablePropertyName);
            }
        }

        private const string UrlToMp3PropertyName = "UrlToMp3";

        private string m_UrlToMp3;
        public string UrlToMp3
        {
            get
            {
                return m_UrlToMp3;
            }
            set
            {
                m_UrlToMp3 = value;
                RaisePropertyChanged(UrlToMp3PropertyName);
            }
        }

        private const string PlayPauseIconUriPropertyName = "PlayPauseIconUri";

        private string m_PlayPauseIconUri;
        public string PlayPauseIconUri
        {
            get
            {
                if (m_PlayPauseIconUri == null)
                {
                    SetToPlay();                 
                }

                return m_PlayPauseIconUri;
            }
            set
            {
                m_PlayPauseIconUri = value;
                RaisePropertyChanged(PlayPauseIconUriPropertyName);
            }
        }

        private const string PlayPauseTextPropertyName = "PlayPauseText";

        private string m_PlayPauseText;
        public string PlayPauseText
        {
            get
            {
                if (m_PlayPauseText == null)
                {
                    m_PlayPauseText = "Play";
                }

                return m_PlayPauseText;
            }
            set
            {
                m_PlayPauseText = value;
                RaisePropertyChanged(PlayPauseTextPropertyName);
            }
        }

        private void SetToPlay()
        {
            if (IsLightThemeEnabled() == false)
            {
                PlayPauseIconUri = "/icons/appbar.transport.play.rest.png";
            }
            else
            {
                PlayPauseIconUri = "/icons/light/appbar.transport.play.rest.png";
            }

            PlayPauseText = "Play";
        }

        private void SetToPause()
        {
            if (IsLightThemeEnabled() == false)
            {
                PlayPauseIconUri = "/icons/appbar.transport.pause.rest.png";
            }
            else
            {
                PlayPauseIconUri = "/icons/light/appbar.transport.pause.rest.png";
            }

            PlayPauseText = "Pause";
        }

        public void PlayPause()
        {
            if (PlayPauseText == "Pause")
            {
                SetToPause();  
            }
            else
            {
                SetToPlay();
            }
        }

        public void SetState(string state)
        {
            if (state == "Playing")
            {
                SetToPause();
                IsPlayable = true;
            }
            else if (state == "Paused")
            {
                SetToPlay();
                IsPlayable = true;
            }
            else if (state == "Stopped")
            {
                SetToPlay();
                IsPlayable = false;
            }
        }
    }
}
