using System;
using GalaSoft.MvvmLight;
using System.Windows.Input;
using System.Collections.ObjectModel;
using System.Collections.Generic;
using Benday.Npr.Common;
using Benday.Npr.Presentation.Interfaces;
using Benday.Npr.Common.DomainModel;
using System.Xml.Serialization;

namespace Benday.Npr.Presentation
{
    public class StoryViewModel : ObjectFactoryViewModel, IStoryViewModel
    {
        public StoryViewModel(IStoryDomainService service, IObjectFactory factory)
            : base(factory)
        {
            if (service == null)
                throw new ArgumentNullException("service", "service is null.");

            m_ServiceInstance = service;
            
            m_Paragraphs = new ObservableCollection<IParagraphViewModel>();
        }

        private IStoryDomainService m_ServiceInstance;
        private IStoryDomainService ServiceInstance
        {
            get { return m_ServiceInstance; }
        }

        public void Load(IStory instance)
        {
            if (instance == null)
                throw new ArgumentNullException("instance", "instance is null.");

            PopulateFromModel(instance);
        }

        private void PopulateFromModel(IStory fromValue)
        {
            this.Id = fromValue.Id;
            this.UrlToHtml = fromValue.UrlToHtml;
            this.UrlToApi = fromValue.UrlToApi;
            this.UrlShort = fromValue.UrlShort;
            this.Title = fromValue.Title;
            this.Teaser = fromValue.Teaser;
            this.TeaserMini = fromValue.TeaserMini;
            this.Slug = fromValue.Slug;
            this.StoryDate = fromValue.StoryDate;
            this.PublishDate = fromValue.PublishDate;
            this.ShowName = fromValue.ShowName;
            this.ShowDate = fromValue.ShowDate;
            this.OrganizationName = fromValue.OrganizationName;
            this.OrganizationAbbreviation = fromValue.OrganizationAbbreviation;
            this.OrganizationUrl = fromValue.OrganizationUrl;
            this.Duration = fromValue.Duration;
            this.UrlToMp3 = fromValue.UrlToMp3;
            this.UrlToWindowsMedia = fromValue.UrlToWindowsMedia;
            this.Author = fromValue.Author;

            m_Paragraphs.Clear();

            IParagraphViewModel toParagraph;

            foreach (var fromParagraph in fromValue.Paragraphs)
            {
                toParagraph = ObjectFactoryInstance.Create<IParagraphViewModel>();

                toParagraph.Text = fromParagraph.Text;

                this.Paragraphs.Add(toParagraph);
            }
        }


        private const string IdPropertyName = "Id";

        private string m_Id;
        public string Id
        {
            get
            {
                return m_Id;
            }
            set
            {
                m_Id = value;
                RaisePropertyChanged(IdPropertyName);
            }
        }

        private const string UrlToHtmlPropertyName = "UrlToHtml";

        private string m_UrlToHtml;
        public string UrlToHtml
        {
            get
            {
                return m_UrlToHtml;
            }
            set
            {
                m_UrlToHtml = value;
                RaisePropertyChanged(UrlToHtmlPropertyName);
            }
        }

        private const string UrlToApiPropertyName = "UrlToApi";

        private string m_UrlToApi;
        public string UrlToApi
        {
            get
            {
                return m_UrlToApi;
            }
            set
            {
                m_UrlToApi = value;
                RaisePropertyChanged(UrlToApiPropertyName);
            }
        }

        private const string UrlShortPropertyName = "UrlShort";

        private string m_UrlShort;
        public string UrlShort
        {
            get
            {
                return m_UrlShort;
            }
            set
            {
                m_UrlShort = value;
                RaisePropertyChanged(UrlShortPropertyName);
            }
        }

        private const string TitlePropertyName = "Title";

        private string m_Title;
        public string Title
        {
            get
            {
                return m_Title;
            }
            set
            {
                m_Title = value;
                RaisePropertyChanged(TitlePropertyName);
            }
        }

        private const string TeaserPropertyName = "Teaser";

        private string m_Teaser;
        public string Teaser
        {
            get
            {
                return m_Teaser;
            }
            set
            {
                m_Teaser = value;
                RaisePropertyChanged(TeaserPropertyName);
            }
        }

        private const string TeaserMiniPropertyName = "TeaserMini";

        private string m_TeaserMini;
        public string TeaserMini
        {
            get
            {
                return m_TeaserMini;
            }
            set
            {
                m_TeaserMini = value;
                RaisePropertyChanged(TeaserMiniPropertyName);
            }
        }

        private const string SlugPropertyName = "Slug";

        private string m_Slug;
        public string Slug
        {
            get
            {
                return m_Slug;
            }
            set
            {
                m_Slug = value;
                RaisePropertyChanged(SlugPropertyName);
            }
        }

        private const string StoryDatePropertyName = "StoryDate";

        private DateTime m_StoryDate;
        public DateTime StoryDate
        {
            get
            {
                return m_StoryDate;
            }
            set
            {
                m_StoryDate = value;
                RaisePropertyChanged(StoryDatePropertyName);
            }
        }

        private const string PublishDatePropertyName = "PublishDate";

        private DateTime m_PublishDate;
        public DateTime PublishDate
        {
            get
            {
                return m_PublishDate;
            }
            set
            {
                m_PublishDate = value;
                RaisePropertyChanged(PublishDatePropertyName);
            }
        }

        private const string ShowNamePropertyName = "ShowName";

        private string m_ShowName;
        public string ShowName
        {
            get
            {
                return m_ShowName;
            }
            set
            {
                m_ShowName = value;
                RaisePropertyChanged(ShowNamePropertyName);
            }
        }

        private const string ShowDatePropertyName = "ShowDate";

        private DateTime m_ShowDate;
        public DateTime ShowDate
        {
            get
            {
                return m_ShowDate;
            }
            set
            {
                m_ShowDate = value;
                RaisePropertyChanged(ShowDatePropertyName);
            }
        }

        private const string OrganizationNamePropertyName = "OrganizationName";

        private string m_OrganizationName;
        public string OrganizationName
        {
            get
            {
                return m_OrganizationName;
            }
            set
            {
                m_OrganizationName = value;
                RaisePropertyChanged(OrganizationNamePropertyName);
            }
        }

        private const string OrganizationAbbreviationPropertyName = "OrganizationAbbreviation";

        private string m_OrganizationAbbreviation;
        public string OrganizationAbbreviation
        {
            get
            {
                return m_OrganizationAbbreviation;
            }
            set
            {
                m_OrganizationAbbreviation = value;
                RaisePropertyChanged(OrganizationAbbreviationPropertyName);
            }
        }

        private const string OrganizationUrlPropertyName = "OrganizationUrl";

        private string m_OrganizationUrl;
        public string OrganizationUrl
        {
            get
            {
                return m_OrganizationUrl;
            }
            set
            {
                m_OrganizationUrl = value;
                RaisePropertyChanged(OrganizationUrlPropertyName);
            }
        }

        private const string DurationPropertyName = "Duration";

        private int m_Duration;
        public int Duration
        {
            get
            {
                return m_Duration;
            }
            set
            {
                m_Duration = value;
                RaisePropertyChanged(DurationPropertyName);
            }
        }

        private const string HasUrlToMp3PropertyName = "HasUrlToMp3";

        public bool HasUrlToMp3
        {
            get
            {
                if (String.IsNullOrEmpty(UrlToMp3) == false)
                {
                    return true;
                }
                else
                {
                    return false;
                }
            }
            set
            {
                RaisePropertyChanged(HasUrlToMp3PropertyName);
            }
        }

        private const string UrlToMp3PropertyName = "UrlToMp3";

        private string m_UrlToMp3;
        public string UrlToMp3
        {
            get
            {
                return m_UrlToMp3;
            }
            set
            {
                m_UrlToMp3 = value;
                RaisePropertyChanged(UrlToMp3PropertyName);
                RaisePropertyChanged(HasUrlToMp3PropertyName);
            }
        }

        private const string UrlToWindowsMediaPropertyName = "UrlToWindowsMedia";

        private string m_UrlToWindowsMedia;
        public string UrlToWindowsMedia
        {
            get
            {
                return m_UrlToWindowsMedia;
            }
            set
            {
                m_UrlToWindowsMedia = value;
                RaisePropertyChanged(UrlToWindowsMediaPropertyName);
                RaisePropertyChanged(HasUrlToWindowsMediaPropertyName);
            }
        }

        private const string HasUrlToWindowsMediaPropertyName = "HasUrlToWindowsMedia";

        public bool HasUrlToWindowsMedia
        {
            get
            {
                if (String.IsNullOrEmpty(UrlToWindowsMedia) == false)
                {
                    return true;
                }
                else
                {
                    return false;
                }
            }
            set
            {
                RaisePropertyChanged(HasUrlToWindowsMediaPropertyName);
            }
        }

        private const string AuthorPropertyName = "Author";

        private string m_Author;
        public string Author
        {
            get
            {
                return m_Author;
            }
            set
            {
                m_Author = value;
                RaisePropertyChanged(AuthorPropertyName);
            }
        }

        private const string ParagraphsPropertyName = "Paragraphs";

        private ObservableCollection<IParagraphViewModel> m_Paragraphs;
        public ObservableCollection<IParagraphViewModel> Paragraphs
        {
            get
            {
                return m_Paragraphs;
            }
            set
            {
                m_Paragraphs = value;
                RaisePropertyChanged(ParagraphsPropertyName);
            }
        }

        public void PlayMp3()
        {
            if (this.HasUrlToMp3 == true)
            {
                if (DoesUrlNeedToBeResolved() == true)
                {
                    ServiceInstance.ResolveMp3UrlFromM3u(new ReturnResult<string>(UrlResolved), this.UrlToMp3);    
                }
                else
                {
                    RaiseOnReadyToPlayMp3();
                }
            }
        }

        private bool DoesUrlNeedToBeResolved()
        {
            if (HasResolvedUrlToMp3 == true)
            {
                return false;
            }
            else
            {
                if (UrlToMp3 != null && UrlToMp3.EndsWith(".mp3") == true)
                {
                    ResolvedUrlToMp3 = UrlToMp3;
                    HasResolvedUrlToMp3 = true;

                    return false;
                }
                else
                {
                    return true;
                }
            }
        }

        private void RaiseOnReadyToPlayMp3()
        {
            if (OnReadyToPlayMp3 != null)
            {
                OnReadyToPlayMp3(this, new EventArgs());
            }
        }

        private void UrlResolved(ReturnResult<string> action)
        {
            ResolvedUrlToMp3 = action.Result;

            if (HasResolvedUrlToMp3 == true)
            {
                RaiseOnReadyToPlayMp3();
            }
        }

        private const string HasResolvedUrlToMp3PropertyName = "HasResolvedUrlToMp3";

        public bool HasResolvedUrlToMp3
        {
            get
            {
                if (String.IsNullOrEmpty(ResolvedUrlToMp3) == false)
                {
                    return true;
                }
                else
                {
                    return false;
                }
            }
            set
            {
                RaisePropertyChanged(HasResolvedUrlToMp3PropertyName);
            }
        }

        private const string ResolvedUrlToMp3PropertyName = "ResolvedUrlToMp3";

        private string m_ResolvedUrlToMp3;
        public string ResolvedUrlToMp3
        {
            get
            {
                return m_ResolvedUrlToMp3;
            }
            set
            {
                m_ResolvedUrlToMp3 = value;
                RaisePropertyChanged(HasResolvedUrlToMp3PropertyName);
                RaisePropertyChanged(ResolvedUrlToMp3PropertyName);
            }
        }


        public event EventHandler OnReadyToPlayMp3;
    }
}
