﻿/*
  In App.xaml:
  <Application.Resources>
      <vm:ViewModelLocator xmlns:vm="clr-namespace:Benday.Npr.Presentation"
                                   x:Key="Locator" />
  </Application.Resources>
  
  In the View:
  DataContext="{Binding Source={StaticResource Locator}, Path=ViewModelName}"
*/

using Benday.Npr.Presentation.Interfaces;
using Benday.Npr.Common;
using System;
using GalaSoft.MvvmLight;
namespace Benday.Npr.Presentation
{
    /// <summary>
    /// This class contains static references to all the view models in the
    /// application and provides an entry point for the bindings.
    /// <para>
    /// Use the <strong>mvvmlocatorproperty</strong> snippet to add ViewModels
    /// to this locator.
    /// </para>
    /// <para>
    /// In Silverlight and WPF, place the ViewModelLocator in the App.xaml resources:
    /// </para>
    /// <code>
    /// &lt;Application.Resources&gt;
    ///     &lt;vm:ViewModelLocator xmlns:vm="clr-namespace:Benday.Npr.Presentation"
    ///                                  x:Key="Locator" /&gt;
    /// &lt;/Application.Resources&gt;
    /// </code>
    /// <para>
    /// Then use:
    /// </para>
    /// <code>
    /// DataContext="{Binding Source={StaticResource Locator}, Path=ViewModelName}"
    /// </code>
    /// <para>
    /// You can also use Blend to do all this with the tool's support.
    /// </para>
    /// <para>
    /// See http://www.galasoft.ch/mvvm/getstarted
    /// </para>
    /// </summary>
    public class ViewModelLocator : ViewModelBase
    {
        /// <summary>
        /// Initializes a new instance of the ViewModelLocator class.
        /// </summary>
        private ViewModelLocator()
        {
            ////if (ViewModelBase.IsInDesignModeStatic)
            ////{
            ////    // Create design time view models
            ////}
            ////else
            ////{
            ////    // Create run time view models
            ////}
        }

        public static void CreateInstance(IObjectFactory factory)
        {
            m_Instance = new ViewModelLocator(factory);                    
        }

        private static ViewModelLocator m_Instance;
        public static ViewModelLocator Instance
        {
            get
            {
                if (m_Instance == null)
                {
                    throw new InvalidOperationException("Not initialized.");
                }

                return m_Instance;
            }
            set
            {
            	m_Instance = value;
            }
        }

        public ViewModelLocator(IObjectFactory factory)
        {
            m_ObjectFactoryInstance = factory;
        }

        private IObjectFactory m_ObjectFactoryInstance;
        public IObjectFactory ObjectFactoryInstance
        {
            get
            {
                return m_ObjectFactoryInstance;
            }
        }

        private IAllProgramsViewModel m_AllPrograms;
        public IAllProgramsViewModel AllPrograms
        {
            get
            {
                if (m_AllPrograms == null)
                {
                    m_AllPrograms = ObjectFactoryInstance.Create<IAllProgramsViewModel>();
                }

                return m_AllPrograms;
            }
        }

        private const string CurrentProgramPropertyName = "CurrentProgram";

        private IProgramViewModel m_CurrentProgram;
        public IProgramViewModel CurrentProgram
        {
            get
            {
                return m_CurrentProgram;
            }
            set
            {
                m_CurrentProgram = value;
                RaisePropertyChanged(CurrentProgramPropertyName);
            }
        }

        private IStoryListViewModel m_StoryListInstance;
        public IStoryListViewModel StoryListInstance
        {
            get
            {
                if (m_StoryListInstance == null)
                {
                    m_StoryListInstance = ObjectFactoryInstance.Create<IStoryListViewModel>();
                }

                return m_StoryListInstance;
            }
        }

        private const string CurrentStoryPropertyName = "CurrentStory";

        private IStoryViewModel m_CurrentStory;
        public IStoryViewModel CurrentStory
        {
            get
            {
                return m_CurrentStory;
            }
            set
            {
                m_CurrentStory = value;
                RaisePropertyChanged(CurrentStoryPropertyName);
            }
        }

        public void NotifyNetworkAvailabilityChange(bool isNetworkAvailable)
        {
            if (this.AllPrograms != null)
            {
                this.AllPrograms.IsNetworkAvailable = isNetworkAvailable;    
            }            

            if (this.CurrentProgram != null)
            {
                this.CurrentProgram.IsNetworkAvailable = isNetworkAvailable;
            }

            if (this.CurrentStory != null)
            {
                this.CurrentStory.IsNetworkAvailable = isNetworkAvailable;
            }

            if (this.StoryListInstance != null)
            {
                this.StoryListInstance.IsNetworkAvailable = isNetworkAvailable;
            }
        }
    }
}