using System;
using System.Collections.Generic;

namespace Benday.Npr.Phone.Common
{
    public class ReturnResult<T>
    {
        /// <summary>
        /// No callback required
        /// </summary>
        public ReturnResult()
        {
            m_callback = null;
        }

        public ReturnResult(Action<ReturnResult<T>> callback)
        {
            if (callback == null)
                throw new ArgumentNullException("callback", "callback is null.");

            m_callback = callback;
        }

        public ReturnResult(Action<ReturnResult<T>> callback, ReturnResult<T> parentCall)
        {
            if (callback == null)
                throw new ArgumentNullException("callback", "callback is null.");
            if (parentCall == null)
                throw new ArgumentNullException("parentCall", "parentCall is null.");

            m_callback = callback;
            ParentCall = parentCall;
        }

        private Action<ReturnResult<T>> m_callback;

        public ReturnResult<T> ParentCall { get; private set; }

        public Exception Error
        {
            get;
            private set;
        }

        public T Result { get; private set; }

        public void Notify(Exception ex)
        {
            Error = ex;
            if (m_callback != null) m_callback(this);
        }

        public void Notify(T result)
        {
            Result = result;
            if (m_callback != null) m_callback(this);
        }

        public void NotifyParentCall()
        {
            if (ParentCall != null)
            {
                if (Error != null)
                {
                    ParentCall.Notify(Error);
                }
                else
                {
                    ParentCall.Notify(Result);
                }
            }
        }
    }
}
