using System;
using System.Linq;
using System.Text;
using System.IO.IsolatedStorage;
using System.IO;
using System.Collections.Generic;

namespace Benday.Npr.Phone.DataAccess
{
    public static class DataAccessUtility
    {
        public static string ReplaceUndisplayableHtml(string original)
        {
            if (original == null)
            {
                return null;
            }
            else
            {
                string cleaned = original;

                cleaned = RemoveComments(cleaned);
                cleaned = cleaned.Replace("<br />", Environment.NewLine);

                StringBuilder builder = new StringBuilder(cleaned);

                builder.Replace("&lt;em&gt;", String.Empty);
                builder.Replace("&lt;/em&gt;", String.Empty);
                builder.Replace("<em>", String.Empty);
                builder.Replace("</em>", String.Empty);
                builder.Replace("&lt;strong&gt;", String.Empty);
                builder.Replace("&lt;/strong&gt;", String.Empty);
                builder.Replace("<strong>", String.Empty);
                builder.Replace("</strong>", String.Empty);
                builder.Replace("</p>", String.Empty);
                builder.Replace("<p>", String.Empty);
                builder.Replace("&#160;", " ");
                builder.Replace("&#8217;", "'");

                cleaned = RemoveHtmlTags(builder.ToString());

                return cleaned.Trim();
            }
        }

        private static string RemoveHtmlTags(string contents)
        {
            if (contents.Contains("<") == true)
            {
                int startOfTag = contents.IndexOf("<");
                int endOfTag = contents.IndexOf(">", startOfTag);

                if (startOfTag != -1 && endOfTag != -1 && (endOfTag - startOfTag + 1) < contents.Length)
                {
                    contents = contents.Remove(startOfTag, (endOfTag - startOfTag + 1));

                    contents = RemoveHtmlTags(contents);    
                }
            }

            return contents;
        }

        private static string RemoveComments(string contents)
        {
            if (contents.Contains("<!--") == true)
            {
                int startOfComment = contents.IndexOf("<!--");
                int endOfComment = contents.IndexOf("-->");

                if (startOfComment != -1 && endOfComment != -1 && (endOfComment + 3) < contents.Length)
                {
                    contents = contents.Remove(startOfComment, (endOfComment - startOfComment + 3));

                    // remove any other comments
                    contents = RemoveComments(contents);
                }
            }

            return contents;
        }

        public static void SaveFile(string filename, System.Xml.Linq.XDocument doc)
        {
            SaveFile(filename, doc.ToString());
        }

        public static void SaveFile(string filename, string contents)
        {
            using (var userIsolatedStorage = IsolatedStorageFile.GetUserStoreForApplication())
            {
                if (userIsolatedStorage.FileExists(filename) == true)
                {
                    userIsolatedStorage.DeleteFile(filename);
                }

                using (var fileStream = userIsolatedStorage.OpenFile(filename, System.IO.FileMode.OpenOrCreate))
                {
                    using (StreamWriter writer = new StreamWriter(fileStream))
                    {
                        writer.Write(contents);
                    }
                }
            }
        }

        public static string ReadFile(string filename)
        {
            using (var userIsolatedStorage = IsolatedStorageFile.GetUserStoreForApplication())
            {
                if (userIsolatedStorage.FileExists(filename) == false)
                {
                    return null;
                }
                else
                {
                    string returnValue = null;

                    using (var fileStream = userIsolatedStorage.OpenFile(filename, System.IO.FileMode.Open))
                    {
                        using (StreamReader reader = new StreamReader(fileStream))
                        {
                            returnValue = reader.ReadToEnd();
                        }
                    }

                    return returnValue;
                }
            }
        }

        public static string GetProgramStoriesFilename(string programId)
        {
            if (String.IsNullOrEmpty(programId))
                throw new ArgumentException("programId is null or empty.", "programId");

            return String.Format(DataAccessConstants.FilenameProgramStoriesList, programId);
        }
    }
}
