﻿using System;
using Benday.Npr.Phone.Common.Repositories;
using Benday.Npr.Phone.Common.Models;
using Benday.Npr.Phone.Common;
using System.Collections.Generic;
using System.Xml.Linq;
using System.Linq;
using System.IO.IsolatedStorage;

namespace Benday.Npr.Phone.DataAccess
{
    public class ProgramRepository : RepositoryBase, IProgramRepository
    {
        public ProgramRepository(IObjectFactory factory, IProgramXmlAdapter adapter)
            : base(factory)
        {
            if (adapter == null)
                throw new ArgumentNullException("adapter", "adapter is null.");

            m_AdapterInstance = adapter;
        }

        private IProgramXmlAdapter m_AdapterInstance;

        public void GetAll(ReturnResult<IList<IProgramInfo>> action)
        {
            if (action == null)
                throw new ArgumentNullException("action", "action is null.");

            List<IProgramInfo> returnValue = GetAll();

            action.Notify(returnValue);
        }

        private List<IProgramInfo> GetAll()
        {
            var allProgramsXml = DataAccessResources.NprAllPrograms;

            List<IProgramInfo> returnValue = new List<IProgramInfo>();

            m_AdapterInstance.Adapt(allProgramsXml, returnValue);
            
            return returnValue;
        }

        public void GetById(ReturnResult<IProgramInfo> action, string id)
        {
            if (action == null)
                throw new ArgumentNullException("action", "action is null.");

            action.Notify(GetById(id));
        }

        public IProgramInfo GetById(string id)
        {
            IProgramInfo returnValue = null;

            var allPrograms = GetAll();

            if (allPrograms != null)
            {
                returnValue = (from temp in allPrograms
                               where temp.Id == id
                               select temp).FirstOrDefault();
            }

            return returnValue;
        }

        public void SaveCurrentProgramId(string currentProgramId)
        {
            var doc = XmlUtility.StringToXDocument("<info />");

            if (currentProgramId == null)
            {
                currentProgramId = String.Empty;
            }
            else
            {
                currentProgramId = currentProgramId.Trim();
            }

            XmlUtility.SetChildElement(doc.Root, "CurrentProgramId", currentProgramId);

            DataAccessUtility.SaveFile(DataAccessConstants.FilenameProgramInformation, doc);
        }

        public string GetCurrentProgramId()
        {
            var contents = DataAccessUtility.ReadFile(DataAccessConstants.FilenameProgramInformation);

            if (contents == null)
            {
                return null;
            }
            else
            {
                var doc = XmlUtility.StringToXDocument(contents);

                string returnValue = 
                    XmlUtility.GetChildElementValue(doc.Root, "CurrentProgramId", null);

                if (returnValue == null || returnValue == String.Empty)
                {
                    return null;
                }
                else
                {
                    return returnValue;    
                }                
            }
        }
    }
}
