using System;
using Benday.Npr.Phone.Common.Repositories;
using Benday.Npr.Phone.Common.Models;
using Benday.Npr.Phone.Common;
using System.Collections.Generic;
using System.Xml.Linq;

namespace Benday.Npr.Phone.DataAccess
{
    public class ProgramXmlAdapter : AdapterBase, IProgramXmlAdapter
    {
        public ProgramXmlAdapter(IObjectFactory factory)
            : base(factory)
        {

        }

        public void Adapt(string fromValue, IList<IProgramInfo> toValue)
        {
            if (String.IsNullOrEmpty(fromValue))
                throw new ArgumentException("fromValue is null or empty.", "fromValue");
            if (toValue == null)
                throw new ArgumentNullException("toValue", "toValue is null.");

            var doc = StringToXDocument(fromValue);

            var root = doc.Root;

            var items = root.Descendants("item");

            IProgramInfo toValueProgramInfo = null;

            foreach (var item in items)
            {
                toValueProgramInfo = ObjectFactoryInstance.Create<IProgramInfo>();

                Adapt(item, toValueProgramInfo);

                toValue.Add(toValueProgramInfo);
            }
        }

        public void Adapt(XElement fromValue, IProgramInfo toValue)
        {
            if (fromValue == null)
                throw new ArgumentNullException("fromValue", "fromValue is null.");
            if (toValue == null)
                throw new ArgumentNullException("toValue", "toValue is null.");

            toValue.Id = XmlUtility.GetAttributeValue(fromValue, "id", null);
            toValue.Name = DataAccessUtility.ReplaceUndisplayableHtml(XmlUtility.GetChildElementValue(fromValue, "title", String.Empty));
            toValue.Description = DataAccessUtility.ReplaceUndisplayableHtml(
                XmlUtility.GetChildElementValue(fromValue, "additionalInfo", String.Empty));
        }
    }
}