﻿using System;
using System.Collections.Generic;
using Benday.Npr.Phone.Common;
using Benday.Npr.Phone.DataAccess;
using Benday.Npr.Phone.Presentation;
using Benday.Npr.Phone.Common.Repositories;
using Benday.Npr.DataAccess;

namespace Benday.Npr.Phone.Factories
{
    public class ObjectFactory : IObjectFactory
    {
        private ObjectFactoryConfiguration m_additionalConfiguration;

        public ObjectFactory()
        {
            m_additionalConfiguration = null;
        }

        public ObjectFactory(ObjectFactoryConfiguration configuration)
        {
            if (configuration == null)
            {
                throw new ArgumentNullException("configuration", "Argument cannot be null.");
            }

            m_additionalConfiguration = configuration;

            InitializeContainer("ObjectFactory");
        }

        private void InitializeContainer(string containerConfigName)
        {
            if (m_additionalConfiguration != null)
            {
                RegisterAdditionalConfiguration(m_additionalConfiguration);
            }
        }

        private void RegisterAdditionalConfiguration(ObjectFactoryConfiguration configuration)
        {
            if (configuration == null)
                throw new ArgumentNullException("configuration", "Configuration is null.");

            foreach (Type key in configuration.Instances.Keys)
            {
                RegisterInstance(key, configuration.Instances[key]);
            }
        }

        private Dictionary<Type, object> m_Instances;
        public Dictionary<Type, object> Instances
        {
            get
            {
                if (m_Instances == null)
                {
                    m_Instances = new Dictionary<Type, object>();
                }

                return m_Instances;
            }
        }        

        private void RegisterInstance(Type interfaceType, object instance)
        {
            if (Instances.ContainsKey(interfaceType) == true)
            {
                Instances.Remove(interfaceType);
            }

            Instances.Add(interfaceType, instance);
        }

        public T Create<T>() where T : class
        {
            var interfaceType = typeof(T);

            if (1 != 1)
            {
                 
            }
            else if (interfaceType.Name == "IProgramRepository")
            {
                return new ProgramRepository(this, new ProgramXmlAdapter(this)) as T;
            }
            else if (interfaceType.Name == "IProgramInfo")
            {
                return new ProgramInfo() as T;
            }
            else if (interfaceType.Name == "IProgramsViewModel")
            {
                return new ProgramsViewModel(Create<IProgramRepository>()) as T;
            }
            else if (interfaceType.Name == "IStoryListViewModel")
            {
                return new StoryListViewModel(Create<IProgramRepository>(), Create<IStoryRepository>(), this) as T;
            }
            else if (interfaceType.Name == "IStoryViewModel")
            {
                return new StoryViewModel(Create<IProgramRepository>(), Create<IStoryRepository>(), this) as T;
            }
            else if (interfaceType.Name == "IStory")
            {
                return new Story() as T;
            }
            else if (interfaceType.Name == "IStoryXmlAdapter")
            {
                return new StoryXmlAdapter(this) as T;
            }
            else if (interfaceType.Name == "IParagraph")
            {
                return new Paragraph() as T;
            }
            else if (interfaceType.Name == "IStoryList")
            {
                return new StoryList() as T;
            }
            else if (interfaceType.Name == "IStoryRepository")
            {
                return new StoryRepository(this, this.Create<IStoryXmlAdapter>()) as T;
            }
            else if (interfaceType.Name == "IRssFeedXmlAdapter")
            {
                return new RssFeedXmlAdapter(this) as T;
            }
            else if (interfaceType.Name == "IParagraphViewModel")
            {
                return new ParagraphViewModel() as T;
            }
            /*
        else if (interfaceType.Name == "IFeed")
        {
            return new Feed() as T;
        }
        else if (interfaceType.Name == "IFeedItem")
        {
            return new FeedItem() as T;
        }
        else if (interfaceType.Name == "IFeedDomainService")
        {
            return GetFeedDomainService() as T;
        }
        else if (interfaceType.Name == "IFeedXmlAdapter")
        {
            return new FeedXmlAdapter(this) as T;
        }
        else if (interfaceType.Name == "IAllProgramsViewModel")
        {
            return new AllProgramsViewModel(GetFeedDomainService(), this) as T;
        }
        else if (interfaceType.Name == "IProgramViewModel")
        {
            return new ProgramViewModel() as T;
        }
        else if (interfaceType.Name == "IFeedRepository")
        {
            return new FeedRepository(this) as T;
        }
        else if (interfaceType.Name == "IStoryDomainService")
        {
            return GetStoryDomainService() as T;
        }
        else if (interfaceType.Name == "IStoryListViewModel")
        {
            return new StoryListViewModel(this.Create<IStoryDomainService>(), this) as T;
        }
        else if (interfaceType.Name == "IStoryViewModel")
        {
            return new StoryViewModel(Create<IStoryDomainService>(), this) as T;
        }
        else if (interfaceType.Name == "IMediaPlaybackViewModel")
        {
            return new MediaPlaybackViewModel() as T;
        }
        else if (interfaceType.Name == "IRssFeedXmlAdapter")
        {
            return new RssFeedXmlAdapter(this) as T;
        }
        else if (interfaceType.Name == "IRssFeedItemList")
        {
            return new RssFeedItemList() as T;
        }
        else if (interfaceType.Name == "IRssFeedItem")
        {
            return new RssFeedItem() as T;
        } 
             */
            else
            {
                throw new InvalidOperationException(String.Format("Type '{0}' is unsupported.", interfaceType.FullName));
            }
        }

        public void Reconfigure(string configurationName)
        {
            InitializeContainer(configurationName);
        }
    }
}
