﻿using System;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Ink;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using GalaSoft.MvvmLight;
using Benday.Npr.Phone.Presentation.Interfaces;
using System.Collections.Generic;
using Benday.Npr.Phone.Common.Repositories;
using Benday.Npr.Phone.Common;
using Benday.Npr.Phone.Common.Models;

namespace Benday.Npr.Phone.Presentation
{
    public class ProgramsViewModel : NetworkAvailabilityViewModel, IProgramsViewModel
    {
        public ProgramsViewModel(IProgramRepository repository)
        {
            if (repository == null)
                throw new ArgumentNullException("repository", "repository is null.");

            m_RepositoryInstance = repository;

            m_Programs = new SingleSelectListViewModel(new List<ISelectableItem>());
            m_Programs.OnItemSelected += new EventHandler(m_Programs_OnItemSelected);
        }

        private IProgramRepository m_RepositoryInstance;
        public IProgramRepository RepositoryInstance
        {
            get { return m_RepositoryInstance; }
        }

        void m_Programs_OnItemSelected(object sender, EventArgs e)
        {
            HandleProgramSelected();
        }

        private void HandleProgramSelected()
        {
            if (Programs.SelectedItem == null)
            {
                CurrentProgramId = null;
                RepositoryInstance.SaveCurrentProgramId(null);
            }
            else
            {
                CurrentProgramId = Programs.SelectedItem.Id;
                RepositoryInstance.SaveCurrentProgramId(CurrentProgramId);
            }            
        }

        private const string CurrentProgramIdPropertyName = "CurrentProgramId";

        private string m_CurrentProgramId;
        public string CurrentProgramId
        {
            get
            {
                return m_CurrentProgramId;
            }
            set
            {
                m_CurrentProgramId = value;
                RaisePropertyChanged(CurrentProgramIdPropertyName);
            }
        }
        
        private const string CurrentProgramPropertyName = "CurrentProgram";

        private const string ProgramsPropertyName = "Programs";

        private SingleSelectListViewModel m_Programs;
        public SingleSelectListViewModel Programs
        {
            get
            {
                return m_Programs;
            }
            set
            {
                m_Programs = value;
                RaisePropertyChanged(ProgramsPropertyName);
            }
        }

        public void Load()
        {
            RepositoryInstance.GetAll(new ReturnResult<IList<IProgramInfo>>(LoadCompleted));
            CurrentProgramId = RepositoryInstance.GetCurrentProgramId();
        }

        private void LoadCompleted(ReturnResult<IList<IProgramInfo>> result)
        {
            Programs.Items.Clear();

            if (result == null || result.Error != null || result.Result == null)
            {
                RequestMessageBox("No results.");
            }
            else
            {
                var toValues = new List<ISelectableItem>();
                ISelectableItem toValue = null;

                foreach (var fromValue in result.Result)
                {
                    toValue = new SelectableItem(false, 
                                  fromValue.Id, 
                                  fromValue.Name, 
                                  fromValue.Id, 
                                  fromValue.Description);

                    toValues.Add(toValue);
                }

                Programs.Initialize(toValues);
            }
        }
    }
}