using System;
using GalaSoft.MvvmLight;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using Benday.Npr.Phone.Presentation.Interfaces;
using Benday.Npr.Phone.Common;

namespace Benday.Npr.Phone.Presentation
{
    public class SelectableItem : ViewModelBase, ISelectableItem
    {
        public SelectableItem()
        {

        }

        /// <summary>
        /// Initializes a new instance of the SelectableItem class.
        /// </summary>
        /// <param name="isSelected"></param>
        /// <param name="text"></param>
        public SelectableItem(bool isSelected, string text)
            : this(isSelected, text, text)
        {

        }

        /// <summary>
        /// Initializes a new instance of the SelectableItemViewModel class.
        /// </summary>
        /// <param name="isSelected"></param>
        /// <param name="text"></param>
        /// <param name="value"></param>
        public SelectableItem(bool isSelected, string text, string value)
        {
            m_IsSelected = isSelected;
            m_Text = text;
            m_Value = value;
        }

        public SelectableItem(bool isSelected, string id, string text, string value, string description)
        {
            m_Id = id;
            m_IsSelected = isSelected;
            m_Text = text;
            m_Value = value;
            m_Description = description;
        }

        private const string IsSelectedPropertyName = "IsSelected";

        private bool m_IsSelected;
        public bool IsSelected
        {
            get
            {
                return m_IsSelected;
            }
            set
            {
                if (m_IsSelected != value)
                {
                    m_IsSelected = value;
                    RaisePropertyChanged(IsSelectedPropertyName);    
                }                
            }
        }

        private const string TextPropertyName = "Text";

        private string m_Text;
        public string Text
        {
            get
            {
                return m_Text;
            }
            set
            {
                m_Text = value;
                RaisePropertyChanged(TextPropertyName);
            }
        }

        private const string ValuePropertyName = "Value";

        private string m_Value;
        public string Value
        {
            get
            {
                return m_Value;
            }
            set
            {
                m_Value = value;
                RaisePropertyChanged(ValuePropertyName);
            }
        }

        public override string ToString()
        {
            return Text;
        }

        private const string DescriptionPropertyName = "Description";

        private string m_Description;
        public string Description
        {
            get
            {
                return m_Description;
            }
            set
            {
                m_Description = value;
                RaisePropertyChanged(DescriptionPropertyName);
            }
        }

        private const string IdPropertyName = "Id";

        private string m_Id;
        public string Id
        {
            get
            {
                return m_Id;
            }
            set
            {
                m_Id = value;
                RaisePropertyChanged(IdPropertyName);
            }
        }
    }
}
