using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Collections.ObjectModel;
using System.Windows.Input;
using GalaSoft.MvvmLight;
using Benday.Npr.Phone.Presentation.Interfaces;
using System.Collections.Specialized;
using System.ComponentModel;
using Benday.Npr.Phone.Common;

namespace Benday.Npr.Phone.Presentation
{
    public class SingleSelectListViewModel : SelectableCollectionViewModel<ISelectableItem>
    {        
        protected SingleSelectListViewModel() : base()
        {
            IsVisible = true;
        }

        public SingleSelectListViewModel(IEnumerable<ISelectableItem> values)
            : this(new ObservableCollection<ISelectableItem>(values))
        {
            if (values == null)
                throw new ArgumentNullException("values", "values is null.");

            IsVisible = true;
        }

        public SingleSelectListViewModel(IEnumerable<ISelectableItem> values, ISelectableItem selectedItem)
            : this(new ObservableCollection<ISelectableItem>(values), selectedItem)
        {
            if (values == null)
                throw new ArgumentNullException("values", "values is null.");
            if (selectedItem == null)
                throw new ArgumentNullException("selectedItem", "selectedItem is null.");

            IsVisible = true;
        }

        public SingleSelectListViewModel(ObservableCollection<ISelectableItem> values) : base(values)
        {
            IsVisible = true;
        }

        public SingleSelectListViewModel(ObservableCollection<ISelectableItem> values, ISelectableItem selectedItem)
            : base(values, selectedItem)
        {
            IsVisible = true;
        }

        public override void Initialize(IEnumerable<ISelectableItem> values)
        {
            Items = new ObservableCollection<ISelectableItem>(values);

            RefreshSelectedItem();

            IsVisible = true;
        }

        public void SelectByText(string text)
        {
            SelectedItem = GetByText(Items, text);
        }

        private ISelectableItem GetByText(ObservableCollection<ISelectableItem> values, string text)
        {
            var selected = (from temp in values
                            where temp.Text == text
                            select temp).FirstOrDefault();

            return selected;
        }

        protected ISelectableItem GetByValue(ObservableCollection<ISelectableItem> values, string value)
        {
            var selected = (from temp in values
                            where temp.Value == value
                            select temp).FirstOrDefault();

            return selected;
        }

        public virtual void SelectByValue(string value)
        {
            SelectedItem = GetByValue(Items, value);
        }

        public void SelectByValue(int value)
        {
            SelectedItem = GetByValue(Items, value.ToString());
        }

        private const string IsVisiblePropertyName = "IsVisible";

        private bool m_IsVisible;
        public bool IsVisible
        {
            get
            {
                return m_IsVisible;
            }
            set
            {
                m_IsVisible = value;
                RaisePropertyChanged(IsVisiblePropertyName);
            }
        }

        private const string IsValidPropertyName = "IsValid";

        private bool m_IsValid;
        public bool IsValid
        {
            get
            {
                return m_IsValid;
            }
            set
            {
                m_IsValid = value;
                RaisePropertyChanged(IsValidPropertyName);
            }
        }

        private const string ValidationMessagePropertyName = "ValidationMessage";

        private string m_ValidationMessage;
        public string ValidationMessage
        {
            get
            {
                return m_ValidationMessage;
            }
            set
            {
                m_ValidationMessage = value;
                RaisePropertyChanged(ValidationMessagePropertyName);
            }
        }

    }
}
