using System;
using System.Collections.Generic;
using System.Linq;
using GalaSoft.MvvmLight;
using Benday.Npr.Phone.Common;
using Benday.Npr.Phone.Presentation.Interfaces;

namespace Benday.Npr.Phone.Presentation
{
    public class ViewModelLocator : ViewModelBase
    {
        /// <summary>
        /// Initializes a new instance of the ViewModelLocator class.
        /// </summary>
        private ViewModelLocator()
        {
            ////if (ViewModelBase.IsInDesignModeStatic)
            ////{
            ////    // Create design time view models
            ////}
            ////else
            ////{
            ////    // Create run time view models
            ////}
        }

        public static void CreateInstance(IObjectFactory factory)
        {
            m_Instance = new ViewModelLocator(factory);
            m_Instance.NotifyNetworkAvailabilityChange(true);
        }

        public static void DestroyInstance()
        {
            m_Instance = null;
        }

        private IStoryViewModel m_Story;
        private IStoryListViewModel m_StoryList;
        private static ViewModelLocator m_Instance;
        public static ViewModelLocator Instance
        {
            get
            {
                if (m_Instance == null)
                {
                    throw new InvalidOperationException("Not initialized.");
                }

                return m_Instance;
            }
            set
            {
                m_Instance = value;
            }
        }

        public ViewModelLocator(IObjectFactory factory)
        {
            m_ObjectFactoryInstance = factory;
        }

        private IObjectFactory m_ObjectFactoryInstance;
        public IObjectFactory ObjectFactoryInstance
        {
            get
            {
                return m_ObjectFactoryInstance;
            }
        }

        private IProgramsViewModel m_Programs;
        public IProgramsViewModel Programs
        {
            get
            {
                if (m_Programs == null)
                {
                    m_Programs = ObjectFactoryInstance.Create<IProgramsViewModel>();
                }

                return m_Programs;
            }
        }

        public IStoryListViewModel StoryList
        {
            get
            {
                if (m_StoryList == null)
                {
                    m_StoryList = ObjectFactoryInstance.Create<IStoryListViewModel>();
                }

                return m_StoryList;
            }
            set
            {
                m_StoryList = value;
            }
        }

        public IStoryViewModel Story
        {
            get
            {
                if (m_Story == null)
                {
                    m_Story = ObjectFactoryInstance.Create<IStoryViewModel>();
                }
                return m_Story;
            }
            set
            {
                m_Story = value;
            }
        }
        
        public void NotifyNetworkAvailabilityChange(bool isNetworkAvailable)
        {
            Programs.IsNetworkAvailable = true;
            StoryList.IsNetworkAvailable = true;
            Story.IsNetworkAvailable = true;
        }

        private const string HasHandledNavigateFromPlaybackAgentPropertyName = "HasHandledNavigateFromPlaybackAgent";

        private bool m_HasHandledNavigateFromPlaybackAgent;
        public bool HasHandledNavigateFromPlaybackAgent
        {
            get
            {
                return m_HasHandledNavigateFromPlaybackAgent;
            }
            set
            {
                m_HasHandledNavigateFromPlaybackAgent = value;
                RaisePropertyChanged(HasHandledNavigateFromPlaybackAgentPropertyName);
            }
        }
    }
}
