using System;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using System.Xml.Linq;
using System.Linq;
using Microsoft.Silverlight.Testing;
using Benday.Npr.Phone.DataAccess;

namespace Benday.Npr.Phone.UnitTests.DataAccess
{
    [TestClass]
    [Tag("UndisplayableHtml")]
    public class DataAccessUtilityFixture
    {
        public DataAccessUtilityFixture()
        {
        }

        [TestMethod]
        public void ReplaceUndisplayableHtml_NoUndisplayableHtmlReturnsOriginal()
        {
            string checkThisString = "clean string";

            var actual = DataAccessUtility.ReplaceUndisplayableHtml(checkThisString);

            Assert.AreEqual<string>(checkThisString, actual, "Should be the same.");

            AssertDoesNotContainUndisplayableHtml(actual);
        }

        [TestMethod]
        public void ReplaceUndisplayableHtml_IfOriginalIsNullReturnsNull()
        {
            string checkThisString = null;

            var actual = DataAccessUtility.ReplaceUndisplayableHtml(checkThisString);

            Assert.AreEqual<string>(checkThisString, actual, "Should be the same.");
        }

        [TestMethod]
        public void ReplaceUndisplayableHtml_EscapedEmphasisTag_WithContent()
        {
            string checkThisString = "This is some text &lt;em&gt;asdf&lt;/em&gt; more text.";

            Assert.IsTrue(UnitTestUtility.StringContainsUndisplayableHtmlFormatting(checkThisString));

            var actual = DataAccessUtility.ReplaceUndisplayableHtml(checkThisString);

            AssertDoesNotContainUndisplayableHtml(actual);

            Assert.AreEqual<string>("This is some text asdf more text.", actual, "Should be the same.");
        }

        [TestMethod]
        public void ReplaceUndisplayableHtml_EscapedEmphasisTag_WithoutContent()
        {
            string checkThisString = "This is some text &lt;em&gt;&lt;/em&gt; more text.";

            Assert.IsTrue(UnitTestUtility.StringContainsUndisplayableHtmlFormatting(checkThisString));

            var actual = DataAccessUtility.ReplaceUndisplayableHtml(checkThisString);

            AssertDoesNotContainUndisplayableHtml(actual);

            Assert.AreEqual<string>("This is some text  more text.", actual, "Should be the same.");
        }

        [TestMethod]
        public void ReplaceUndisplayableHtml_UnescapedEmphasisTag_WithContent()
        {
            string checkThisString = "This is some text <em>asdf</em> more text.";

            Assert.IsTrue(UnitTestUtility.StringContainsUndisplayableHtmlFormatting(checkThisString));

            var actual = DataAccessUtility.ReplaceUndisplayableHtml(checkThisString);

            AssertDoesNotContainUndisplayableHtml(actual);

            Assert.AreEqual<string>("This is some text asdf more text.", actual, "Should be the same.");
        }

        [TestMethod]
        public void ReplaceUndisplayableHtml_UnescapedEmphasisTag_WithoutContent()
        {
            string checkThisString = "This is some text <em></em> more text.";

            Assert.IsTrue(UnitTestUtility.StringContainsUndisplayableHtmlFormatting(checkThisString));

            var actual = DataAccessUtility.ReplaceUndisplayableHtml(checkThisString);

            AssertDoesNotContainUndisplayableHtml(actual);

            Assert.AreEqual<string>("This is some text  more text.", actual, "Should be the same.");
        }

        [TestMethod]
        public void ReplaceUndisplayableHtml_UnescapedStrongTag_WithContent()
        {
            string checkThisString = "This is some text <strong>asdf</strong> more text.";

            Assert.IsTrue(UnitTestUtility.StringContainsUndisplayableHtmlFormatting(checkThisString));

            var actual = DataAccessUtility.ReplaceUndisplayableHtml(checkThisString);

            AssertDoesNotContainUndisplayableHtml(actual);

            Assert.AreEqual<string>("This is some text asdf more text.", actual, "Should be the same.");
        }

        [TestMethod]
        public void ReplaceUndisplayableHtml_UnescapedStrongTag_WithoutContent()
        {
            string checkThisString = "This is some text <strong></strong> more text.";

            Assert.IsTrue(UnitTestUtility.StringContainsUndisplayableHtmlFormatting(checkThisString));

            var actual = DataAccessUtility.ReplaceUndisplayableHtml(checkThisString);

            AssertDoesNotContainUndisplayableHtml(actual);

            Assert.AreEqual<string>("This is some text  more text.", actual, "Should be the same.");
        }

        private void AssertDoesNotContainUndisplayableHtml(string actual)
        {
            Assert.IsFalse(
                UnitTestUtility.StringContainsUndisplayableHtmlFormatting(actual), 
                "Should not contain any undisplayable html.  ('{0}')", actual);
        }
    }
}
