using System;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using System.Xml.Linq;
using System.Linq;
using System.IO.IsolatedStorage;

namespace Benday.Npr.Phone.UnitTests.DataAccess
{
    public class UnitTestUtility
    {
        public UnitTestUtility()
        {

        }

        public static void AssertNotNullOrEmpty(string actual, string fieldName)
        {
            Assert.AreNotEqual<string>(null, actual, "'{0}' should not be null.", fieldName);
            Assert.AreNotEqual<string>(String.Empty, actual, "'{0}' should not be an empty string.", fieldName);
        }

        public static void AssertStringDoesNotContainUndisplayableHtmlFormatting(string checkThisString)
        {
            Assert.IsFalse(StringContainsUndisplayableHtmlFormatting(checkThisString), 
                "Should not contain any undisplayable html formatting.  ('{0}')", 
                checkThisString);
        }

        public static void AssertStringDoesNotContainUndisplayableHtmlFormatting(string checkThisString, string message)
        {
            Assert.IsFalse(StringContainsUndisplayableHtmlFormatting(checkThisString),
                "{1} should not contain any undisplayable html formatting.  ('{0}')",
                checkThisString, message);
        }

        public static bool StringContainsUndisplayableHtmlFormatting(string checkThisString)
        {
            Assert.IsNotNull(checkThisString, "String was null.");

            if (checkThisString.Contains("&lt;em&gt;") == true || 
                checkThisString.Contains("&lt;/em&gt;") == true || 
                checkThisString.Contains("<em>") == true || 
                checkThisString.Contains("</em>") == true || 
                checkThisString.Contains("<strong>") == true || 
                checkThisString.Contains("</strong>") == true)
            {
                return true;
            }
            else
            {
                return false;
            }
        }

        public static void WipeProgramDataInIsolatedStorage()
        {
            using (var userIsolatedStorage = IsolatedStorageFile.GetUserStoreForApplication())
            {
                var files = userIsolatedStorage.GetFileNames();

                foreach (var file in files)
                {
                    userIsolatedStorage.DeleteFile(file);
                }
            }
        }

        public static bool FileExists(string filename)
        {
            using (var userIsolatedStorage = IsolatedStorageFile.GetUserStoreForApplication())
            {
                var files = userIsolatedStorage.GetFileNames();

                if (files == null)
                {
                    return false;
                }
                else
                {
                    var match = (from temp in files
                                 where temp == filename
                                 select temp).FirstOrDefault();

                    if (match == null)
                    {
                        return false;
                    }
                    else
                    {
                        return true;
                    }
                }
            }
        }
    }
}
