using System;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using System.Xml.Linq;
using Microsoft.Silverlight.Testing;
using Benday.Npr.Phone.Common;

namespace Benday.Npr.Phone.UnitTests.DataAccess
{
    [TestClass]
    [Tag("xml")]
    public class XmlUtilityFixture : UnitTestBase
    {
        [TestMethod]
        public void XmlUtility_GetChildElementValueByElementNameAndAttributeId()
        {
            string xml = TestDataUtility.XmlWithSimilarChildElementsWithDifferentAttributes();

            // <link type="html">http://www.npr.org/2010/11/10/131216486/in-israel-when-is-a-jew-not-jewish-enough?ft=3&amp;f=2</link>

            string expected = "http://www.npr.org/2010/11/10/131216486/in-israel-when-is-a-jew-not-jewish-enough?ft=3&f=2";

            var root = XmlUtility.StringToXDocument(xml);

            var actual = XmlUtility.GetChildElementValue(root.Root, "link", "type", "html", String.Empty);

            Assert.AreEqual<string>(expected, actual, "Value was wrong.");
        }

        [TestMethod]
        public void GetAttributeValueAsInt32_ReturnsIntForValidValue()
        {
            var expected = 1234;

            XElement element = new XElement("element");
            XAttribute attribute = new XAttribute("attr", expected.ToString());
            element.Add(attribute);
                        
            var actual = XmlUtility.GetAttributeValueAsInt32(element, "attr", -1);

            Assert.AreEqual<int>(expected, actual, "Got the wrong value.");
        }

        [TestMethod]
        public void GetAttributeValueAsInt32_ReturnsIntForInvalidValue()
        {
            var expected = -1;

            XElement element = new XElement("element");
            XAttribute attribute = new XAttribute("attr", "this is not an int32");
            element.Add(attribute);

            var actual = XmlUtility.GetAttributeValueAsInt32(element, "attr", expected);

            Assert.AreEqual<int>(expected, actual, "Got the wrong value.");
        }
        
    }
}
