using System;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Benday.Npr.Phone.Presentation.Interfaces;
using Benday.Npr.Phone.UnitTests.DataAccess;
using Benday.Npr.Phone.Common.Repositories;
using Benday.Npr.Phone.DataAccess;
using Benday.Npr.Phone.Presentation;

namespace Benday.Npr.Phone.UnitTests.Presentation
{
    [TestClass]
    public class StoryViewModelFixture : UnitTestBase
    {
        protected override void OnTestInitialize()
        {
            UnitTestUtility.WipeProgramDataInIsolatedStorage();
            m_ViewModelInstance = null;
            m_RepositoryInstance = null;
            m_ProgramRepositoryInstance = null;
            base.OnTestInitialize();
        }

        private IStoryViewModel m_ViewModelInstance;
        public IStoryViewModel ViewModelInstance
        {
            get
            {
                if (m_ViewModelInstance == null)
                {
                    m_ViewModelInstance = new StoryViewModel(
                        ProgramRepositoryInstance,
                        RepositoryInstance,
                        ObjectFactoryInstance);
                }

                return m_ViewModelInstance;
            }
        }

        private StoryRepository m_RepositoryInstance;
        public StoryRepository RepositoryInstance
        {
            get
            {
                if (m_RepositoryInstance == null)
                {
                    m_RepositoryInstance =
                        ObjectFactoryInstance.Create<IStoryRepository>() as StoryRepository;

                    m_RepositoryInstance.TestModeConfiguration =
                        new StoryRepositoryTestModeConfiguration() { TestData = UnitTestResources.AllThingsConsideredStories };
                }

                return m_RepositoryInstance;
            }
        }

        private IProgramRepository m_ProgramRepositoryInstance;
        public IProgramRepository ProgramRepositoryInstance
        {
            get
            {
                if (m_ProgramRepositoryInstance == null)
                {
                    m_ProgramRepositoryInstance =
                        new ProgramRepositoryFake();
                }

                return m_ProgramRepositoryInstance;
            }
        }

        [TestMethod]
        public void CreateInstanceOfStoryViewModelViaObjectFactory()
        {
            var item = ObjectFactoryInstance.Create<IStoryViewModel>();

            Assert.IsNotNull(item);
        }

        [TestMethod]
        public void LoadFromCacheForCurrentStoryId()
        {
            // force the repository to write the test data to cache
            RepositoryInstance.RefreshStories("2");
            string expectedStoryId = "131216486";
            string expectedStoryTitle = "In Israel, When Is A Jew Not Jewish Enough?";

            RepositoryInstance.SaveCurrentStoryId(expectedStoryId);

            ViewModelInstance.Load();

            UnitTestUtility.AssertNotNullOrEmpty(ViewModelInstance.CurrentProgramId, "CurrentProgramId");
            
            Assert.AreEqual<string>(expectedStoryId, ViewModelInstance.Id, "Story Id was wrong.");
            Assert.AreEqual<string>(expectedStoryTitle, ViewModelInstance.Title, "Story title was wrong.");
        }

    }
}
