using System;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Benday.Npr.Phone.Common;

namespace Benday.Npr.Phone.UnitTests
{
    [TestClass]
    public class ReturnResultFixture : UnitTestBase
    {
        public ReturnResultFixture()
        {
        }

        [TestMethod]
        public void ReturnResult_Notify_Result()
        {
            bool gotCall = false;

            string expectedResultValue = "expected result";

            var instance = new ReturnResult<string>(new Action<ReturnResult<string>>(
                arg =>
                {
                    gotCall = true;
                    Assert.IsNull(arg.Error, "Error should be null.");
                    Assert.AreEqual<string>(expectedResultValue, arg.Result, "Result was wrong.");
                    Assert.IsNull(arg.ParentCall, "ParentCall should be null.");
                }));

            instance.Notify(expectedResultValue);

            Assert.IsTrue(gotCall, "Did not receive call.");
        }

        [TestMethod]
        public void ReturnResult_Notify_Error()
        {
            bool gotCall = false;

            ArgumentException expectedErrorValue = new ArgumentException("BOOM!");

            var instance = new ReturnResult<string>(new Action<ReturnResult<string>>(
                arg =>
                {
                    gotCall = true;
                    Assert.IsNull(arg.Result, "Result should be null.");
                    Assert.AreSame(expectedErrorValue, arg.Error, "Error instance was wrong.");
                    Assert.IsNull(arg.ParentCall, "ParentCall should be null.");
                }));

            instance.Notify(expectedErrorValue);

            Assert.IsTrue(gotCall, "Did not receive call.");
        }

        [TestMethod]
        public void ReturnResult_NotifyParent_Success()
        {
            bool gotCall = false;
            bool gotParentCall = false;

            string expectedResultValue = "expected result";

            var parentCall = new ReturnResult<string>(new Action<ReturnResult<string>>(
                arg =>
                {
                    gotParentCall = true;
                    Assert.IsNull(arg.Error, "Error should be null.");
                    Assert.AreEqual<string>(expectedResultValue, arg.Result, "Result was wrong.");
                    Assert.IsNull(arg.ParentCall, "ParentCall should be null.");
                }));

            var instance = new ReturnResult<string>(new Action<ReturnResult<string>>(
                arg =>
                {
                    gotCall = true;
                    Assert.IsNull(arg.Error, "Error should be null.");
                    Assert.AreEqual<string>(expectedResultValue, arg.Result, "Result was wrong.");
                    Assert.IsNotNull(arg.ParentCall, "ParentCall should not be null.");
                }), parentCall);

            instance.Notify(expectedResultValue);

            // forward the same result to the parent call
            instance.NotifyParentCall();

            Assert.IsTrue(gotCall, "Did not receive call.");
            Assert.IsTrue(gotParentCall, "Did not receive parent call.");
        }

        [TestMethod]
        public void ReturnResult_NotifyParent_Error()
        {
            bool gotCall = false;
            bool gotParentCall = false;

            ArgumentException expectedErrorValue = new ArgumentException("BOOM!");

            var parentCall = new ReturnResult<string>(new Action<ReturnResult<string>>(
                arg =>
                {
                    gotParentCall = true;
                    Assert.IsNull(arg.Result, "Result should be null.");
                    Assert.AreSame(expectedErrorValue, arg.Error, "Error instance was wrong.");
                    Assert.IsNull(arg.ParentCall, "ParentCall should be null.");
                }));

            var instance = new ReturnResult<string>(new Action<ReturnResult<string>>(
                arg =>
                {
                    gotCall = true;
                    Assert.IsNull(arg.Result, "Result should be null.");
                    Assert.AreSame(expectedErrorValue, arg.Error, "Error instance was wrong.");
                    Assert.IsNotNull(arg.ParentCall, "ParentCall should be null.");
                }), parentCall);

            instance.Notify(expectedErrorValue);

            // forward the same result to the parent call
            instance.NotifyParentCall();

            Assert.IsTrue(gotCall, "Did not receive call.");
            Assert.IsTrue(gotParentCall, "Did not receive parent call.");
        }
    }
}
