﻿using System;
using Microsoft.Silverlight.Testing;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Benday.Npr.Phone.Factories;
using Benday.Npr.Phone.Common;

namespace Benday.Npr.Phone.UnitTests
{
    public abstract class UnitTestBase : SilverlightTest
    {
        private ObjectFactoryConfiguration m_additionalObjectFactoryConfiguration;
        public ObjectFactoryConfiguration AdditionalObjectFactoryConfiguration
        {
            get
            {
                if (m_additionalObjectFactoryConfiguration == null)
                {
                    m_additionalObjectFactoryConfiguration = new ObjectFactoryConfiguration();

                    PopulateAdditionalObjectFactoryConfiguration(m_additionalObjectFactoryConfiguration);
                }

                return m_additionalObjectFactoryConfiguration;
            }
        }

        protected virtual void PopulateAdditionalObjectFactoryConfiguration(ObjectFactoryConfiguration configuration)
        {
            
        }

        private IObjectFactory m_objectFactoryInstance;
        public IObjectFactory ObjectFactoryInstance
        {
            get
            {
                if (m_objectFactoryInstance == null)
                {
                    m_objectFactoryInstance = new ObjectFactory(AdditionalObjectFactoryConfiguration);                    
                }

                return m_objectFactoryInstance;
            }
        }

        public void ResetObjectFactoryInstance()
        {
            m_objectFactoryInstance = null;    
        }

        protected virtual void OnTestInitialize()
        {
            m_objectFactoryInstance = null;
            m_additionalObjectFactoryConfiguration = null;
        }

        [TestInitialize]
        public void TestInitialize()
        {
            OnTestInitialize();
        }

        protected virtual void OnClassInitialize()
        {
            
        }

        [ClassInitialize]
        public void ClassInitialize()
        {
        	OnClassInitialize();
        }
    }
}