﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using Microsoft.Phone.Controls;
using Microsoft.Phone.Tasks;
using Benday.Npr.Phone.Presentation;
using Benday.Npr.Phone.Presentation.Interfaces;
using Microsoft.Phone.BackgroundAudio;

namespace Benday.Npr.Phone
{
    public partial class MainPage : PhoneApplicationPage
    {
        // Constructor
        public MainPage()
        {
            InitializeComponent();
            
            this.Loaded += new RoutedEventHandler(MainPage_Loaded);

            PopulateVersionNumber();
        }

        private void PopulateVersionNumber()
        {
            m_textVersion.Text = "v2.0.0";
        }

        private IProgramsViewModel m_ViewModel;
        private void MainPage_Loaded(object sender, RoutedEventArgs e)
        {
            m_ViewModel = ViewModelLocator.Instance.Programs;

            this.DataContext = m_ViewModel;

            m_ViewModel.Load();

            m_ViewModel.Programs.OnItemSelected += new EventHandler(Programs_OnItemSelected);

            CheckIfPlayingAudioAndNavigateIfRequired();
        }

        private void CheckIfPlayingAudioAndNavigateIfRequired()
        {
            if (ViewModelLocator.Instance.HasHandledNavigateFromPlaybackAgent == true)
            {
                return;
            }

            if (BackgroundAudioPlayer.Instance != null && BackgroundAudioPlayer.Instance.Track != null)
            {
                var track = BackgroundAudioPlayer.Instance.Track;

                if (String.IsNullOrWhiteSpace(track.Tag) == false && track.Tag.StartsWith("nprlistener:") == true)
                {
                    string tag = track.Tag;

                    tag = tag.Replace("nprlistener:", String.Empty);

                    var tokens = tag.Split(new string[] { ":" }, StringSplitOptions.None);

                    if (tokens.Length == 2)
                    {
                        string programId = tokens[0];
                        string storyId = tokens[1];

                        ViewModelLocator.Instance.Programs.Programs.SelectByValue(programId);
                    }
                }
            }
        }

        void Programs_OnItemSelected(object sender, EventArgs e)
        {
            NavigateToProgram();
        }

        private void NavigateToProgram()
        {
            if (m_ViewModel != null && m_ViewModel.Programs.SelectedItem != null)
            {
                NavigationService.Navigate(new Uri("/StoryListPage.xaml", UriKind.Relative));

                /*
                if (m_ViewModel.Programs.SelectedItem.Id == "RSS_FEED.NPR_HOURLY_NEWS")
                {
                    NavigationService.Navigate(new Uri("/HourlyNewsPage.xaml", UriKind.Relative));
                }
                else
                {
                    NavigationService.Navigate(new Uri("/StoryListPage.xaml", UriKind.Relative));
                }
                */
            }
        }

        private void HyperlinkButton_Click(object sender, RoutedEventArgs e)
        {
            HyperlinkButton senderAsHyperLink = sender as HyperlinkButton;

            if (senderAsHyperLink != null)
            {
                var task = new WebBrowserTask();
                task.Uri = new Uri(senderAsHyperLink.Tag as string);

                task.Show();
            }
        }
    }
}