﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using Microsoft.Phone.Controls;
using Benday.Npr.Phone.Presentation.Interfaces;
using Benday.Npr.Phone.Presentation;
using Microsoft.Phone.BackgroundAudio;
using Microsoft.Xna.Framework.Media;
using System.Threading;

namespace Benday.Npr.Phone
{
    public partial class StoryDetailsPage : PhoneApplicationPage
    {
        public StoryDetailsPage()
        {
            InitializeComponent();
        }

        private IStoryViewModel m_ViewModel;

        private void PhoneApplicationPage_Loaded(object sender, RoutedEventArgs e)
        {
            m_ViewModel = ViewModelLocator.Instance.Story;

            m_ViewModel.OnReadyToPlayMp3 += new EventHandler(m_ViewModel_OnReadyToPlayMp3);

            this.DataContext = m_ViewModel;

            m_ViewModel.Load();

            m_Watcher = new NavigateToHomeApplicationBarWatcher(
                ApplicationBar.Buttons[0] as Microsoft.Phone.Shell.ApplicationBarIconButton);
        }

        private NavigateToHomeApplicationBarWatcher m_Watcher;

        private void m_HomeButton_Click(object sender, EventArgs e)
        {
            NavigationService.Navigate(new Uri("/MainPage.xaml", UriKind.Relative));
        }

        void m_ViewModel_OnReadyToPlayMp3(object sender, EventArgs e)
        {
            try
            {
                PlayTrack(m_ViewModel);
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.ToString());
            } 
        }

        private void PlayTrack(IStoryViewModel viewModel)
        {
            Uri uri = new Uri(viewModel.ResolvedUrlToMp3, UriKind.RelativeOrAbsolute);

            AudioTrack track = new AudioTrack(uri,
                                   viewModel.Title,
                                   viewModel.ShowName,
                                   viewModel.PublishDate.ToShortDateString(),
                                   null, viewModel.AudioTrackLocatorTag, EnabledPlayerControls.All);

            BackgroundAudioPlayer.Instance.Track = track;
            BackgroundAudioPlayer.Instance.Play();
        }

        private void PlayTrack()
        {
            PlayTrack(m_ViewModel);
        }

        private void CheckPlaybackStateAndOptionallyPlayTrack()
        {
            if (MediaPlayer.GameHasControl == false && MediaPlayer.State == MediaState.Playing)
            {
                var result = MessageBox.Show("Would you like to stop the current music playback?",
                                 "Stop current track?",
                                 MessageBoxButton.OKCancel);

                if (result == MessageBoxResult.OK)
                {
                    m_ViewModel.PlayMp3();
                }
            }
            else
            {
                m_ViewModel.PlayMp3();
            }
        }

        private void m_BtnPlay_Click(object sender, RoutedEventArgs e)
        {
            try
            {
                if (m_ViewModel == null)
                {
                    return;
                }
                else
                {
                    CheckPlaybackStateAndOptionallyPlayTrack();
                }
                
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.ToString());
            }  
        }
    }
}