﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using Microsoft.Phone.Controls;
using Benday.Npr.Phone.Presentation.Interfaces;
using Benday.Npr.Phone.Presentation;
using Microsoft.Phone.BackgroundAudio;

namespace Benday.Npr.Phone
{
    public partial class StoryListPage : PhoneApplicationPage
    {
        public StoryListPage()
        {
            InitializeComponent();
        }

        private IStoryListViewModel m_ViewModel;

        private void PhoneApplicationPage_Loaded(object sender, RoutedEventArgs e)
        {
            m_ViewModel = ViewModelLocator.Instance.StoryList;

            this.DataContext = m_ViewModel;

            m_ViewModel.Load();

            m_ViewModel.Stories.OnItemSelected += new EventHandler(Stories_OnItemSelected);

            CheckIfPlayingAudioAndNavigateIfRequired();
            
            m_WatcherNavigateToHome = new NavigateToHomeApplicationBarWatcher(
                ApplicationBar.Buttons[0] as Microsoft.Phone.Shell.ApplicationBarIconButton);

            m_WatcherRefresh = new RefreshStoriesApplicationBarWatcher(
                ApplicationBar.Buttons[1] as Microsoft.Phone.Shell.ApplicationBarIconButton);
        }

        private NavigateToHomeApplicationBarWatcher m_WatcherNavigateToHome;
        private RefreshStoriesApplicationBarWatcher m_WatcherRefresh;

        private void CheckIfPlayingAudioAndNavigateIfRequired()
        {
            if (ViewModelLocator.Instance.HasHandledNavigateFromPlaybackAgent == true)
            {
                return;
            }

            if (BackgroundAudioPlayer.Instance != null && BackgroundAudioPlayer.Instance.Track != null)
            {
                var track = BackgroundAudioPlayer.Instance.Track;

                if (String.IsNullOrWhiteSpace(track.Tag) == false && track.Tag.StartsWith("nprlistener:") == true)
                {
                    string tag = track.Tag;

                    tag = tag.Replace("nprlistener:", String.Empty);

                    var tokens = tag.Split(new string[] { ":" }, StringSplitOptions.None);

                    if (tokens.Length == 2)
                    {
                        string programId = tokens[0];
                        string storyId = tokens[1];

                        ViewModelLocator.Instance.HasHandledNavigateFromPlaybackAgent = true;

                        ViewModelLocator.Instance.StoryList.Stories.SelectByValue(storyId);                        
                    }
                }
            }
        }

        private void m_HomeButton_Click(object sender, EventArgs e)
        {
            NavigationService.Navigate(new Uri("/MainPage.xaml", UriKind.Relative));
        }

        private void m_RefreshButton_Click(object sender, EventArgs e)
        {
            m_ViewModel.Refresh();
        }

        void Stories_OnItemSelected(object sender, EventArgs e)
        {
            NavigateToStory();
        }

        private void NavigateToStory()
        {
            if (m_ViewModel != null && m_ViewModel.Stories.SelectedItem != null)
            {
                NavigationService.Navigate(new Uri("/StoryDetailsPage.xaml", UriKind.Relative));
            }
        }        
    }
}